import base64
import hashlib
from typing import Callable, Optional, Union

import bech32
import coincurve
import ecdsa
from ecdsa.curves import Curve
from ecdsa.util import sigencode_string, sigencode_string_canonize

from neurionpy.crypto.interface import Signer


def _base64_decode(value: str) -> bytes:
    try:
        return base64.b64decode(value)
    except Exception as error:
        raise RuntimeError("Unable to parse base64 value") from error


class PublicKey:
    """Public key class."""

    curve: Curve = ecdsa.SECP256k1
    hash_function: Callable = hashlib.sha256

    def __init__(self, public_key: Union[bytes, "PublicKey", ecdsa.VerifyingKey]):
        """Initialize.

        :param public_key: butes, public key or ecdsa verifying key instance
        :raises RuntimeError: Invalid public key
        """
        if isinstance(public_key, bytes):
            self._verifying_key = ecdsa.VerifyingKey.from_string(
                public_key, curve=self.curve, hashfunc=self.hash_function
            )
        elif isinstance(public_key, PublicKey):
            self._verifying_key = public_key._verifying_key
        elif isinstance(public_key, ecdsa.VerifyingKey):
            self._verifying_key = public_key
        else:
            raise RuntimeError("Invalid public key type")  # noqa

        self._public_key_bytes: bytes = self._verifying_key.to_string("compressed")
        self._public_key: str = base64.b64encode(self._public_key_bytes).decode()

    @property
    def public_key(self) -> str:
        """
        Get public key.

        :return: str public key.
        """
        return self._public_key

    @property
    def public_key_hex(self) -> str:
        """
        Get public key hex.

        :return: str public key hex.
        """
        return self.public_key_bytes.hex()

    @property
    def public_key_bytes(self) -> bytes:
        """
        Get bytes public key.

        :return: bytes public key.
        """
        return self._public_key_bytes

    def verify(self, message: bytes, signature: bytes) -> bool:
        """
        Verify message and signature.

        :param message: bytes message content.
        :param signature: bytes signature.
        :return: bool is message and signature valid.
        """
        success: bool = False

        try:
            success = self._verifying_key.verify(signature, message)

        except ecdsa.keys.BadSignatureError:
            ...

        return success

    def verify_digest(self, digest: bytes, signature: bytes) -> bool:
        """
        Verify digest.

        :param digest: bytes digest.
        :param signature: bytes signature.
        :return: bool is digest valid.
        """
        success: bool = False

        try:
            success = self._verifying_key.verify_digest(signature, digest)

        except ecdsa.keys.BadSignatureError:  # pragma: no cover
            ...

        return success

    def to_address(self, prefix: str = "neurion") -> str:
        """
        Converts a public key in hex format to a Cosmos SDK-compatible Bech32 address.

        :param public_key_hex: Hex-encoded public key (compressed format).
        :param prefix: Bech32 address prefix (default is "neurion").
        :return: Cosmos SDK-compatible address.
        """
        # Step 1: Decode hex public key into bytes
        public_key_bytes = bytes.fromhex(self.public_key_hex)

        # Step 2: Perform SHA-256 hash on public key
        sha256_hash = hashlib.sha256(public_key_bytes).digest()

        # Step 3: Perform RIPEMD-160 hash on the SHA-256 result
        ripemd160_hash = hashlib.new("ripemd160", sha256_hash).digest()

        # Step 4: Encode into Bech32 format with "neurion" prefix
        address = bech32.bech32_encode(prefix, bech32.convertbits(ripemd160_hash, 8, 5))

        return address

    @staticmethod
    def from_signature(message: bytes, signature: bytes) -> "PublicKey":
        """
        Recover the public key from a signed message.

        :param message: The original message that was signed.
        :param signature: The 65-byte signature containing the recovery id.
        :return: The recovered PublicKey instance.
        """
        message_hash = hashlib.sha256(message).digest()
        recovered_key = coincurve.PublicKey.from_signature_and_message(signature, message_hash)
        return PublicKey(recovered_key.format(compressed=True))

    @staticmethod
    def from_signature_hex(message: str, signature_hex: str) -> "PublicKey":
        """
        Recover the public key from a signed message.

        :param message: The original message that was signed.
        :param signature: The 65-byte signature containing the recovery id.
        :return: The recovered PublicKey instance.
        """
        message = message.encode()
        message_hash = hashlib.sha256(message).digest()
        signature = bytes.fromhex(signature_hex)
        recovered_key = coincurve.PublicKey.from_signature_and_message(signature, message_hash)
        return PublicKey(recovered_key.format(compressed=True))


class PrivateKey(Signer):
    """Private key class."""

    curve: Curve = ecdsa.SECP256k1
    hash_function: Callable = hashlib.sha256

    def __init__(self, private_key: Optional[Union[bytes, str]] = None):
        """
        Initialize.

        :param private_key: bytes private key (optional, None by default).
        :raises RuntimeError: if unable to load private key from input.
        """
        if private_key is None:
            self._signing_key = ecdsa.SigningKey.generate(
                curve=self.curve, hashfunc=self.hash_function
            )
        elif isinstance(private_key, bytes):
            self._signing_key = ecdsa.SigningKey.from_string(
                private_key, curve=self.curve, hashfunc=self.hash_function
            )
        elif isinstance(private_key, str):
            raw_private_key = _base64_decode(private_key)
            self._signing_key = ecdsa.SigningKey.from_string(
                raw_private_key, curve=self.curve, hashfunc=self.hash_function
            )

        else:
            raise RuntimeError("Unable to load private key from input")

        # cache the binary representations of the private key
        self._private_key_bytes = self._signing_key.to_string()
        self._private_key = base64.b64encode(self._private_key_bytes).decode()

    @property
    def private_key(self) -> str:
        """
        Get private key.

        :return: str private key.
        """
        return self._private_key

    @property
    def private_key_hex(self) -> str:
        """
        Get private key hex.

        :return: str private key hex.
        """
        return self.private_key_bytes.hex()

    @property
    def private_key_bytes(self) -> bytes:
        """
        Get bytes private key.

        :return: bytes private key.
        """
        return self._private_key_bytes

    @property
    def public_key(self) -> PublicKey:
        """
        Get public key.

        :return: public key.
        """
        return PublicKey(self._signing_key.get_verifying_key())

    def sign(
        self, message: bytes, deterministic: bool = True, canonicalise: bool = True
    ) -> bytes:
        """
        Sign message.

        :param message: bytes message content.
        :param deterministic: bool is deterministic.
        :param canonicalise: bool is canonicalise.

        :return: bytes signed message.
        """
        sigencode = sigencode_string_canonize if canonicalise else sigencode_string
        sign_fnc = (
            self._signing_key.sign_deterministic
            if deterministic
            else self._signing_key.sign
        )

        return sign_fnc(message, sigencode=sigencode)

    def sign_recoverable(self, message: bytes) -> bytes:
        """
        Sign message using a recoverable signature.

        :param message: bytes message content.
        :return: bytes signed message.
        """
        message_hash = hashlib.sha256(message).digest()
        return coincurve.PrivateKey(self._private_key_bytes).sign_recoverable(message_hash)

    def sign_digest(
        self, digest: bytes, deterministic=True, canonicalise: bool = True
    ) -> bytes:
        """
        Sign digest.

        :param digest: bytes digest content.
        :param deterministic: bool is deterministic.
        :param canonicalise: bool is canonicalise.

        :return: bytes signed digest.
        """
        sigencode = sigencode_string_canonize if canonicalise else sigencode_string
        sign_fnc = (
            self._signing_key.sign_digest_deterministic
            if deterministic
            else self._signing_key.sign_digest
        )

        return sign_fnc(digest, sigencode=sigencode)



from typing import Optional

from neurionpy.protos.neurion.sanctum.tx_pb2 import (
    MsgUpdateParams,
    MsgUpdateParamsResponse,
    MsgSubmitDatasetApplication,
    MsgSubmitDatasetApplicationResponse,
    MsgApproveApplication,
    MsgApproveApplicationResponse,
    MsgRejectApplication,
    MsgRejectApplicationResponse,
    MsgDisclaimDataset,
    MsgDisclaimDatasetResponse,
    MsgRequestToUseDataset,
    MsgRequestToUseDatasetResponse,
    MsgCancelDatasetUsageRequest,
    MsgCancelDatasetUsageRequestResponse,
    MsgRejectDatasetUsageRequest,
    MsgRejectDatasetUsageRequestResponse,
    MsgApproveDatasetUsageRequest,
    MsgApproveDatasetUsageRequestResponse,
    MsgAddProcessor,
    MsgAddProcessorResponse,
    MsgRemoveProcessor,
    MsgRemoveProcessorResponse,
    MsgProcessDatasetUsageRequest,
    MsgProcessDatasetUsageRequestResponse,
    MsgFinishDatasetUsageRequest,
    MsgFinishDatasetUsageRequestResponse,
    MsgDisputeDatasetUsageRequest,
    MsgDisputeDatasetUsageRequestResponse,
    MsgApproveDispute,
    MsgApproveDisputeResponse,
    MsgRejectDispute,
    MsgRejectDisputeResponse,
    MsgStakeToSanctum,
    MsgStakeToSanctumResponse,
    MsgUnstakeFromSanctum,
    MsgUnstakeFromSanctumResponse,
    MsgClaimReward,
    MsgClaimRewardResponse,
)
from neurionpy.sanctum.interface import SanctumMessage
from neurionpy.synapse.tx import Transaction
from neurionpy.synapse.tx_helpers import SubmittedTx
from neurionpy.synapse.utils import NeurionUtils
from neurionpy.synapse.wallet import Wallet


class SanctumMsgClient(SanctumMessage):
    def __init__(self, utils: NeurionUtils, wallet: Optional[Wallet]) -> None:
        """
        Initialize the Sanctum message client.

        :param utils: NeurionUtils instance for transaction preparation and broadcasting.
        :param wallet: Wallet instance for signing transactions.
        :raises ValueError: if wallet is not provided.
        """
        if wallet is None:
            raise ValueError("A wallet must be provided to initialize SanctumMsgClient.")
        self.utils = utils
        self.wallet = wallet

    def _set_creator(self, message) -> None:
        """
        If the message has a 'creator' field and it is not set, assign it to the wallet's address.
        """
        if hasattr(message, "creator") and not bool(message.creator):
            message.creator = str(self.wallet.address())

    def UpdateParams(
            self,
            message: MsgUpdateParams,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Update module parameters.

        Note: This message uses an 'authority' field, so no creator defaulting is applied.
        """
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def SubmitDatasetApplication(
            self,
            message: MsgSubmitDatasetApplication,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Submit a dataset application.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def ApproveApplication(
            self,
            message: MsgApproveApplication,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Approve a dataset application.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def RejectApplication(
            self,
            message: MsgRejectApplication,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Reject a dataset application.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def DisclaimDataset(
            self,
            message: MsgDisclaimDataset,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Disclaim a dataset.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def RequestToUseDataset(
            self,
            message: MsgRequestToUseDataset,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Request to use a dataset.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def CancelDatasetUsageRequest(
            self,
            message: MsgCancelDatasetUsageRequest,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Cancel a dataset usage request.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def RejectDatasetUsageRequest(
            self,
            message: MsgRejectDatasetUsageRequest,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Reject a dataset usage request.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def ApproveDatasetUsageRequest(
            self,
            message: MsgApproveDatasetUsageRequest,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Approve a dataset usage request.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def AddProcessor(
            self,
            message: MsgAddProcessor,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Add a processor.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def RemoveProcessor(
            self,
            message: MsgRemoveProcessor,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Remove a processor.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def ProcessDatasetUsageRequest(
            self,
            message: MsgProcessDatasetUsageRequest,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Process a dataset usage request.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def FinishDatasetUsageRequest(
            self,
            message: MsgFinishDatasetUsageRequest,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Finish a dataset usage request.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def DisputeDatasetUsageRequest(
            self,
            message: MsgDisputeDatasetUsageRequest,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Dispute a dataset usage request.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def ApproveDispute(
            self,
            message: MsgApproveDispute,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Approve a dispute.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def RejectDispute(
            self,
            message: MsgRejectDispute,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Reject a dispute.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def StakeToSanctum(
            self,
            message: MsgStakeToSanctum,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Stake tokens to Sanctum.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def UnstakeFromSanctum(
            self,
            message: MsgUnstakeFromSanctum,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Unstake tokens from Sanctum.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)

    def ClaimReward(
            self,
            message: MsgClaimReward,
            memo: Optional[str] = None,
            gas_limit: Optional[int] = None,
    ) -> SubmittedTx:
        """
        Claim reward.
        """
        self._set_creator(message)
        tx = Transaction()
        tx.add_message(message)
        return self.utils.prepare_and_broadcast_basic_transaction(tx, self.wallet, gas_limit=gas_limit, memo=memo)


from google.protobuf.json_format import Parse

from neurionpy.bank.interface import Bank
from neurionpy.common.rest_client import RestClient
from neurionpy.protos.cosmos.bank.v1beta1.query_pb2 import (
    QueryAllBalancesRequest,
    QueryAllBalancesResponse,
    QueryBalanceRequest,
    QueryBalanceResponse,
    QueryDenomMetadataRequest,
    QueryDenomMetadataResponse,
    QueryDenomsMetadataRequest,
    QueryDenomsMetadataResponse,
    QueryParamsRequest,
    QueryParamsResponse,
    QuerySupplyOfRequest,
    QuerySupplyOfResponse,
    QueryTotalSupplyRequest,
    QueryTotalSupplyResponse,
)


class BankRestClient(Bank):
    """Bank REST client."""

    API_URL = "/cosmos/bank/v1beta1"

    def __init__(self, rest_api: RestClient):
        """
        Create bank rest client.

        :param rest_api: RestClient api
        """
        self._rest_api = rest_api

    def Balance(self, request: QueryBalanceRequest) -> QueryBalanceResponse:
        """
        Query balance of selected denomination from specific account.

        :param request: QueryBalanceRequest with address and denomination

        :return: QueryBalanceResponse
        """
        response = self._rest_api.get(
            f"{self.API_URL}/balances/{request.address}/by_denom?denom={request.denom}",
            request,
            ["address", "denom"],
        )
        return Parse(response, QueryBalanceResponse())

    def AllBalances(self, request: QueryAllBalancesRequest) -> QueryAllBalancesResponse:
        """
        Query balance of all denominations from specific account.

        :param request: QueryAllBalancesRequest with account address

        :return: QueryAllBalancesResponse
        """
        response = self._rest_api.get(
            f"{self.API_URL}/balances/{request.address}", request, ["address"]
        )
        return Parse(response, QueryAllBalancesResponse())

    def TotalSupply(self, request: QueryTotalSupplyRequest) -> QueryTotalSupplyResponse:
        """
        Query total supply of all denominations.

        :param request: QueryTotalSupplyRequest

        :return: QueryTotalSupplyResponse
        """
        response = self._rest_api.get(f"{self.API_URL}/supply", request)
        return Parse(response, QueryTotalSupplyResponse())

    def SupplyOf(self, request: QuerySupplyOfRequest) -> QuerySupplyOfResponse:
        """
        Query total supply of specific denomination.

        :param request: QuerySupplyOfRequest with denomination

        :return: QuerySupplyOfResponse
        """
        response = self._rest_api.get(f"{self.API_URL}/supply/{request.denom}")
        return Parse(response, QuerySupplyOfResponse())

    def Params(self, request: QueryParamsRequest) -> QueryParamsResponse:
        """
        Query the parameters of bank module.

        :param request: QueryParamsRequest

        :return: QueryParamsResponse
        """
        response = self._rest_api.get(f"{self.API_URL}/params")
        return Parse(response, QueryParamsResponse())

    def DenomMetadata(
        self, request: QueryDenomMetadataRequest
    ) -> QueryDenomMetadataResponse:
        """
        Query the client metadata for all registered coin denominations.

        :param request: QueryDenomMetadataRequest with denomination

        :return: QueryDenomMetadataResponse
        """
        response = self._rest_api.get(f"{self.API_URL}/denoms_metadata/{request.denom}")
        return Parse(response, QueryDenomMetadataResponse())

    def DenomsMetadata(
        self, request: QueryDenomsMetadataRequest
    ) -> QueryDenomsMetadataResponse:
        """
        Query the client metadata of a given coin denomination.

        :param request: QueryDenomsMetadataRequest

        :return: QueryDenomsMetadataResponse
        """
        response = self._rest_api.get(f"{self.API_URL}/denoms_metadata", request)
        return Parse(response, QueryDenomsMetadataResponse())



from abc import ABC, abstractmethod

from neurionpy.protos.cosmos.upgrade.v1beta1.query_pb2 import (
    QueryAppliedPlanRequest,
    QueryAppliedPlanResponse,
    QueryCurrentPlanRequest,
    QueryCurrentPlanResponse,
)


class CosmosUpgrade(ABC):
    """Cosmos Upgrade abstract class."""

    @abstractmethod
    def CurrentPlan(self, request: QueryCurrentPlanRequest) -> QueryCurrentPlanResponse:
        """
        CurrentPlan queries the current upgrade plan.

        :param request: QueryCurrentPlanRequest
        :return: QueryCurrentPlanResponse
        """

    @abstractmethod
    def AppliedPlan(self, request: QueryAppliedPlanRequest) -> QueryAppliedPlanResponse:
        """
        AppliedPlan queries a previously applied upgrade plan by its name.

        :param request: QueryAppliedPlanRequest
        :return: QueryAppliedPlanResponse
        """



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: ibc/core/connection/v1/genesis.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'ibc/core/connection/v1/genesis.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from ibc.core.connection.v1 import connection_pb2 as ibc_dot_core_dot_connection_dot_v1_dot_connection__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$ibc/core/connection/v1/genesis.proto\x12\x16ibc.core.connection.v1\x1a\x14gogoproto/gogo.proto\x1a\'ibc/core/connection/v1/connection.proto\"\xc6\x02\n\x0cGenesisState\x12G\n\x0b\x63onnections\x18\x01 \x03(\x0b\x32,.ibc.core.connection.v1.IdentifiedConnectionB\x04\xc8\xde\x1f\x00\x12p\n\x17\x63lient_connection_paths\x18\x02 \x03(\x0b\x32\'.ibc.core.connection.v1.ConnectionPathsB&\xc8\xde\x1f\x00\xf2\xde\x1f\x1eyaml:\"client_connection_paths\"\x12\x45\n\x18next_connection_sequence\x18\x03 \x01(\x04\x42#\xf2\xde\x1f\x1fyaml:\"next_connection_sequence\"\x12\x34\n\x06params\x18\x04 \x01(\x0b\x32\x1e.ibc.core.connection.v1.ParamsB\x04\xc8\xde\x1f\x00\x42>Z<github.com/cosmos/ibc-go/v2/modules/core/03-connection/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ibc.core.connection.v1.genesis_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z<github.com/cosmos/ibc-go/v2/modules/core/03-connection/types'
  _globals['_GENESISSTATE'].fields_by_name['connections']._loaded_options = None
  _globals['_GENESISSTATE'].fields_by_name['connections']._serialized_options = b'\310\336\037\000'
  _globals['_GENESISSTATE'].fields_by_name['client_connection_paths']._loaded_options = None
  _globals['_GENESISSTATE'].fields_by_name['client_connection_paths']._serialized_options = b'\310\336\037\000\362\336\037\036yaml:\"client_connection_paths\"'
  _globals['_GENESISSTATE'].fields_by_name['next_connection_sequence']._loaded_options = None
  _globals['_GENESISSTATE'].fields_by_name['next_connection_sequence']._serialized_options = b'\362\336\037\037yaml:\"next_connection_sequence\"'
  _globals['_GENESISSTATE'].fields_by_name['params']._loaded_options = None
  _globals['_GENESISSTATE'].fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _globals['_GENESISSTATE']._serialized_start=128
  _globals['_GENESISSTATE']._serialized_end=454
# @@protoc_insertion_point(module_scope)



from abc import ABC, abstractmethod

from neurionpy.protos.cosmos.params.v1beta1.query_pb2 import (
    QueryParamsRequest,
    QueryParamsResponse,
)


class Params(ABC):
    """Params abstract class."""

    @abstractmethod
    def Params(self, request: QueryParamsRequest) -> QueryParamsResponse:
        """
        Params queries a specific Cosmos SDK parameter.

        :param request: QueryParamsRequest
        :return: QueryParamsResponse
        """



from cometbft.abci.v1beta1 import types_pb2 as _types_pb2
from cometbft.abci.v1beta2 import types_pb2 as _types_pb2_1
from cometbft.types.v1 import params_pb2 as _params_pb2
from cometbft.types.v1beta1 import validator_pb2 as _validator_pb2
from gogoproto import gogo_pb2 as _gogo_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Request(_message.Message):
    __slots__ = ("echo", "flush", "info", "init_chain", "query", "check_tx", "commit", "list_snapshots", "offer_snapshot", "load_snapshot_chunk", "apply_snapshot_chunk", "prepare_proposal", "process_proposal", "extend_vote", "verify_vote_extension", "finalize_block")
    ECHO_FIELD_NUMBER: _ClassVar[int]
    FLUSH_FIELD_NUMBER: _ClassVar[int]
    INFO_FIELD_NUMBER: _ClassVar[int]
    INIT_CHAIN_FIELD_NUMBER: _ClassVar[int]
    QUERY_FIELD_NUMBER: _ClassVar[int]
    CHECK_TX_FIELD_NUMBER: _ClassVar[int]
    COMMIT_FIELD_NUMBER: _ClassVar[int]
    LIST_SNAPSHOTS_FIELD_NUMBER: _ClassVar[int]
    OFFER_SNAPSHOT_FIELD_NUMBER: _ClassVar[int]
    LOAD_SNAPSHOT_CHUNK_FIELD_NUMBER: _ClassVar[int]
    APPLY_SNAPSHOT_CHUNK_FIELD_NUMBER: _ClassVar[int]
    PREPARE_PROPOSAL_FIELD_NUMBER: _ClassVar[int]
    PROCESS_PROPOSAL_FIELD_NUMBER: _ClassVar[int]
    EXTEND_VOTE_FIELD_NUMBER: _ClassVar[int]
    VERIFY_VOTE_EXTENSION_FIELD_NUMBER: _ClassVar[int]
    FINALIZE_BLOCK_FIELD_NUMBER: _ClassVar[int]
    echo: _types_pb2.RequestEcho
    flush: _types_pb2.RequestFlush
    info: _types_pb2_1.RequestInfo
    init_chain: RequestInitChain
    query: _types_pb2.RequestQuery
    check_tx: _types_pb2.RequestCheckTx
    commit: _types_pb2.RequestCommit
    list_snapshots: _types_pb2.RequestListSnapshots
    offer_snapshot: _types_pb2.RequestOfferSnapshot
    load_snapshot_chunk: _types_pb2.RequestLoadSnapshotChunk
    apply_snapshot_chunk: _types_pb2.RequestApplySnapshotChunk
    prepare_proposal: RequestPrepareProposal
    process_proposal: RequestProcessProposal
    extend_vote: RequestExtendVote
    verify_vote_extension: RequestVerifyVoteExtension
    finalize_block: RequestFinalizeBlock
    def __init__(self, echo: _Optional[_Union[_types_pb2.RequestEcho, _Mapping]] = ..., flush: _Optional[_Union[_types_pb2.RequestFlush, _Mapping]] = ..., info: _Optional[_Union[_types_pb2_1.RequestInfo, _Mapping]] = ..., init_chain: _Optional[_Union[RequestInitChain, _Mapping]] = ..., query: _Optional[_Union[_types_pb2.RequestQuery, _Mapping]] = ..., check_tx: _Optional[_Union[_types_pb2.RequestCheckTx, _Mapping]] = ..., commit: _Optional[_Union[_types_pb2.RequestCommit, _Mapping]] = ..., list_snapshots: _Optional[_Union[_types_pb2.RequestListSnapshots, _Mapping]] = ..., offer_snapshot: _Optional[_Union[_types_pb2.RequestOfferSnapshot, _Mapping]] = ..., load_snapshot_chunk: _Optional[_Union[_types_pb2.RequestLoadSnapshotChunk, _Mapping]] = ..., apply_snapshot_chunk: _Optional[_Union[_types_pb2.RequestApplySnapshotChunk, _Mapping]] = ..., prepare_proposal: _Optional[_Union[RequestPrepareProposal, _Mapping]] = ..., process_proposal: _Optional[_Union[RequestProcessProposal, _Mapping]] = ..., extend_vote: _Optional[_Union[RequestExtendVote, _Mapping]] = ..., verify_vote_extension: _Optional[_Union[RequestVerifyVoteExtension, _Mapping]] = ..., finalize_block: _Optional[_Union[RequestFinalizeBlock, _Mapping]] = ...) -> None: ...

class RequestInitChain(_message.Message):
    __slots__ = ("time", "chain_id", "consensus_params", "validators", "app_state_bytes", "initial_height")
    TIME_FIELD_NUMBER: _ClassVar[int]
    CHAIN_ID_FIELD_NUMBER: _ClassVar[int]
    CONSENSUS_PARAMS_FIELD_NUMBER: _ClassVar[int]
    VALIDATORS_FIELD_NUMBER: _ClassVar[int]
    APP_STATE_BYTES_FIELD_NUMBER: _ClassVar[int]
    INITIAL_HEIGHT_FIELD_NUMBER: _ClassVar[int]
    time: _timestamp_pb2.Timestamp
    chain_id: str
    consensus_params: _params_pb2.ConsensusParams
    validators: _containers.RepeatedCompositeFieldContainer[_types_pb2.ValidatorUpdate]
    app_state_bytes: bytes
    initial_height: int
    def __init__(self, time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., chain_id: _Optional[str] = ..., consensus_params: _Optional[_Union[_params_pb2.ConsensusParams, _Mapping]] = ..., validators: _Optional[_Iterable[_Union[_types_pb2.ValidatorUpdate, _Mapping]]] = ..., app_state_bytes: _Optional[bytes] = ..., initial_height: _Optional[int] = ...) -> None: ...

class RequestPrepareProposal(_message.Message):
    __slots__ = ("max_tx_bytes", "txs", "local_last_commit", "misbehavior", "height", "time", "next_validators_hash", "proposer_address")
    MAX_TX_BYTES_FIELD_NUMBER: _ClassVar[int]
    TXS_FIELD_NUMBER: _ClassVar[int]
    LOCAL_LAST_COMMIT_FIELD_NUMBER: _ClassVar[int]
    MISBEHAVIOR_FIELD_NUMBER: _ClassVar[int]
    HEIGHT_FIELD_NUMBER: _ClassVar[int]
    TIME_FIELD_NUMBER: _ClassVar[int]
    NEXT_VALIDATORS_HASH_FIELD_NUMBER: _ClassVar[int]
    PROPOSER_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    max_tx_bytes: int
    txs: _containers.RepeatedScalarFieldContainer[bytes]
    local_last_commit: ExtendedCommitInfo
    misbehavior: _containers.RepeatedCompositeFieldContainer[_types_pb2_1.Misbehavior]
    height: int
    time: _timestamp_pb2.Timestamp
    next_validators_hash: bytes
    proposer_address: bytes
    def __init__(self, max_tx_bytes: _Optional[int] = ..., txs: _Optional[_Iterable[bytes]] = ..., local_last_commit: _Optional[_Union[ExtendedCommitInfo, _Mapping]] = ..., misbehavior: _Optional[_Iterable[_Union[_types_pb2_1.Misbehavior, _Mapping]]] = ..., height: _Optional[int] = ..., time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., next_validators_hash: _Optional[bytes] = ..., proposer_address: _Optional[bytes] = ...) -> None: ...

class RequestProcessProposal(_message.Message):
    __slots__ = ("txs", "proposed_last_commit", "misbehavior", "hash", "height", "time", "next_validators_hash", "proposer_address")
    TXS_FIELD_NUMBER: _ClassVar[int]
    PROPOSED_LAST_COMMIT_FIELD_NUMBER: _ClassVar[int]
    MISBEHAVIOR_FIELD_NUMBER: _ClassVar[int]
    HASH_FIELD_NUMBER: _ClassVar[int]
    HEIGHT_FIELD_NUMBER: _ClassVar[int]
    TIME_FIELD_NUMBER: _ClassVar[int]
    NEXT_VALIDATORS_HASH_FIELD_NUMBER: _ClassVar[int]
    PROPOSER_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    txs: _containers.RepeatedScalarFieldContainer[bytes]
    proposed_last_commit: CommitInfo
    misbehavior: _containers.RepeatedCompositeFieldContainer[_types_pb2_1.Misbehavior]
    hash: bytes
    height: int
    time: _timestamp_pb2.Timestamp
    next_validators_hash: bytes
    proposer_address: bytes
    def __init__(self, txs: _Optional[_Iterable[bytes]] = ..., proposed_last_commit: _Optional[_Union[CommitInfo, _Mapping]] = ..., misbehavior: _Optional[_Iterable[_Union[_types_pb2_1.Misbehavior, _Mapping]]] = ..., hash: _Optional[bytes] = ..., height: _Optional[int] = ..., time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., next_validators_hash: _Optional[bytes] = ..., proposer_address: _Optional[bytes] = ...) -> None: ...

class RequestExtendVote(_message.Message):
    __slots__ = ("hash", "height", "time", "txs", "proposed_last_commit", "misbehavior", "next_validators_hash", "proposer_address")
    HASH_FIELD_NUMBER: _ClassVar[int]
    HEIGHT_FIELD_NUMBER: _ClassVar[int]
    TIME_FIELD_NUMBER: _ClassVar[int]
    TXS_FIELD_NUMBER: _ClassVar[int]
    PROPOSED_LAST_COMMIT_FIELD_NUMBER: _ClassVar[int]
    MISBEHAVIOR_FIELD_NUMBER: _ClassVar[int]
    NEXT_VALIDATORS_HASH_FIELD_NUMBER: _ClassVar[int]
    PROPOSER_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    hash: bytes
    height: int
    time: _timestamp_pb2.Timestamp
    txs: _containers.RepeatedScalarFieldContainer[bytes]
    proposed_last_commit: CommitInfo
    misbehavior: _containers.RepeatedCompositeFieldContainer[_types_pb2_1.Misbehavior]
    next_validators_hash: bytes
    proposer_address: bytes
    def __init__(self, hash: _Optional[bytes] = ..., height: _Optional[int] = ..., time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., txs: _Optional[_Iterable[bytes]] = ..., proposed_last_commit: _Optional[_Union[CommitInfo, _Mapping]] = ..., misbehavior: _Optional[_Iterable[_Union[_types_pb2_1.Misbehavior, _Mapping]]] = ..., next_validators_hash: _Optional[bytes] = ..., proposer_address: _Optional[bytes] = ...) -> None: ...

class RequestVerifyVoteExtension(_message.Message):
    __slots__ = ("hash", "validator_address", "height", "vote_extension")
    HASH_FIELD_NUMBER: _ClassVar[int]
    VALIDATOR_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    HEIGHT_FIELD_NUMBER: _ClassVar[int]
    VOTE_EXTENSION_FIELD_NUMBER: _ClassVar[int]
    hash: bytes
    validator_address: bytes
    height: int
    vote_extension: bytes
    def __init__(self, hash: _Optional[bytes] = ..., validator_address: _Optional[bytes] = ..., height: _Optional[int] = ..., vote_extension: _Optional[bytes] = ...) -> None: ...

class RequestFinalizeBlock(_message.Message):
    __slots__ = ("txs", "decided_last_commit", "misbehavior", "hash", "height", "time", "next_validators_hash", "proposer_address")
    TXS_FIELD_NUMBER: _ClassVar[int]
    DECIDED_LAST_COMMIT_FIELD_NUMBER: _ClassVar[int]
    MISBEHAVIOR_FIELD_NUMBER: _ClassVar[int]
    HASH_FIELD_NUMBER: _ClassVar[int]
    HEIGHT_FIELD_NUMBER: _ClassVar[int]
    TIME_FIELD_NUMBER: _ClassVar[int]
    NEXT_VALIDATORS_HASH_FIELD_NUMBER: _ClassVar[int]
    PROPOSER_ADDRESS_FIELD_NUMBER: _ClassVar[int]
    txs: _containers.RepeatedScalarFieldContainer[bytes]
    decided_last_commit: CommitInfo
    misbehavior: _containers.RepeatedCompositeFieldContainer[_types_pb2_1.Misbehavior]
    hash: bytes
    height: int
    time: _timestamp_pb2.Timestamp
    next_validators_hash: bytes
    proposer_address: bytes
    def __init__(self, txs: _Optional[_Iterable[bytes]] = ..., decided_last_commit: _Optional[_Union[CommitInfo, _Mapping]] = ..., misbehavior: _Optional[_Iterable[_Union[_types_pb2_1.Misbehavior, _Mapping]]] = ..., hash: _Optional[bytes] = ..., height: _Optional[int] = ..., time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., next_validators_hash: _Optional[bytes] = ..., proposer_address: _Optional[bytes] = ...) -> None: ...

class Response(_message.Message):
    __slots__ = ("exception", "echo", "flush", "info", "init_chain", "query", "check_tx", "commit", "list_snapshots", "offer_snapshot", "load_snapshot_chunk", "apply_snapshot_chunk", "prepare_proposal", "process_proposal", "extend_vote", "verify_vote_extension", "finalize_block")
    EXCEPTION_FIELD_NUMBER: _ClassVar[int]
    ECHO_FIELD_NUMBER: _ClassVar[int]
    FLUSH_FIELD_NUMBER: _ClassVar[int]
    INFO_FIELD_NUMBER: _ClassVar[int]
    INIT_CHAIN_FIELD_NUMBER: _ClassVar[int]
    QUERY_FIELD_NUMBER: _ClassVar[int]
    CHECK_TX_FIELD_NUMBER: _ClassVar[int]
    COMMIT_FIELD_NUMBER: _ClassVar[int]
    LIST_SNAPSHOTS_FIELD_NUMBER: _ClassVar[int]
    OFFER_SNAPSHOT_FIELD_NUMBER: _ClassVar[int]
    LOAD_SNAPSHOT_CHUNK_FIELD_NUMBER: _ClassVar[int]
    APPLY_SNAPSHOT_CHUNK_FIELD_NUMBER: _ClassVar[int]
    PREPARE_PROPOSAL_FIELD_NUMBER: _ClassVar[int]
    PROCESS_PROPOSAL_FIELD_NUMBER: _ClassVar[int]
    EXTEND_VOTE_FIELD_NUMBER: _ClassVar[int]
    VERIFY_VOTE_EXTENSION_FIELD_NUMBER: _ClassVar[int]
    FINALIZE_BLOCK_FIELD_NUMBER: _ClassVar[int]
    exception: _types_pb2.ResponseException
    echo: _types_pb2.ResponseEcho
    flush: _types_pb2.ResponseFlush
    info: _types_pb2.ResponseInfo
    init_chain: ResponseInitChain
    query: _types_pb2.ResponseQuery
    check_tx: ResponseCheckTx
    commit: ResponseCommit
    list_snapshots: _types_pb2.ResponseListSnapshots
    offer_snapshot: _types_pb2.ResponseOfferSnapshot
    load_snapshot_chunk: _types_pb2.ResponseLoadSnapshotChunk
    apply_snapshot_chunk: _types_pb2.ResponseApplySnapshotChunk
    prepare_proposal: _types_pb2_1.ResponsePrepareProposal
    process_proposal: _types_pb2_1.ResponseProcessProposal
    extend_vote: ResponseExtendVote
    verify_vote_extension: ResponseVerifyVoteExtension
    finalize_block: ResponseFinalizeBlock
    def __init__(self, exception: _Optional[_Union[_types_pb2.ResponseException, _Mapping]] = ..., echo: _Optional[_Union[_types_pb2.ResponseEcho, _Mapping]] = ..., flush: _Optional[_Union[_types_pb2.ResponseFlush, _Mapping]] = ..., info: _Optional[_Union[_types_pb2.ResponseInfo, _Mapping]] = ..., init_chain: _Optional[_Union[ResponseInitChain, _Mapping]] = ..., query: _Optional[_Union[_types_pb2.ResponseQuery, _Mapping]] = ..., check_tx: _Optional[_Union[ResponseCheckTx, _Mapping]] = ..., commit: _Optional[_Union[ResponseCommit, _Mapping]] = ..., list_snapshots: _Optional[_Union[_types_pb2.ResponseListSnapshots, _Mapping]] = ..., offer_snapshot: _Optional[_Union[_types_pb2.ResponseOfferSnapshot, _Mapping]] = ..., load_snapshot_chunk: _Optional[_Union[_types_pb2.ResponseLoadSnapshotChunk, _Mapping]] = ..., apply_snapshot_chunk: _Optional[_Union[_types_pb2.ResponseApplySnapshotChunk, _Mapping]] = ..., prepare_proposal: _Optional[_Union[_types_pb2_1.ResponsePrepareProposal, _Mapping]] = ..., process_proposal: _Optional[_Union[_types_pb2_1.ResponseProcessProposal, _Mapping]] = ..., extend_vote: _Optional[_Union[ResponseExtendVote, _Mapping]] = ..., verify_vote_extension: _Optional[_Union[ResponseVerifyVoteExtension, _Mapping]] = ..., finalize_block: _Optional[_Union[ResponseFinalizeBlock, _Mapping]] = ...) -> None: ...

class ResponseInitChain(_message.Message):
    __slots__ = ("consensus_params", "validators", "app_hash")
    CONSENSUS_PARAMS_FIELD_NUMBER: _ClassVar[int]
    VALIDATORS_FIELD_NUMBER: _ClassVar[int]
    APP_HASH_FIELD_NUMBER: _ClassVar[int]
    consensus_params: _params_pb2.ConsensusParams
    validators: _containers.RepeatedCompositeFieldContainer[_types_pb2.ValidatorUpdate]
    app_hash: bytes
    def __init__(self, consensus_params: _Optional[_Union[_params_pb2.ConsensusParams, _Mapping]] = ..., validators: _Optional[_Iterable[_Union[_types_pb2.ValidatorUpdate, _Mapping]]] = ..., app_hash: _Optional[bytes] = ...) -> None: ...

class ResponseCheckTx(_message.Message):
    __slots__ = ("code", "data", "log", "info", "gas_wanted", "gas_used", "events", "codespace")
    CODE_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    LOG_FIELD_NUMBER: _ClassVar[int]
    INFO_FIELD_NUMBER: _ClassVar[int]
    GAS_WANTED_FIELD_NUMBER: _ClassVar[int]
    GAS_USED_FIELD_NUMBER: _ClassVar[int]
    EVENTS_FIELD_NUMBER: _ClassVar[int]
    CODESPACE_FIELD_NUMBER: _ClassVar[int]
    code: int
    data: bytes
    log: str
    info: str
    gas_wanted: int
    gas_used: int
    events: _containers.RepeatedCompositeFieldContainer[_types_pb2_1.Event]
    codespace: str
    def __init__(self, code: _Optional[int] = ..., data: _Optional[bytes] = ..., log: _Optional[str] = ..., info: _Optional[str] = ..., gas_wanted: _Optional[int] = ..., gas_used: _Optional[int] = ..., events: _Optional[_Iterable[_Union[_types_pb2_1.Event, _Mapping]]] = ..., codespace: _Optional[str] = ...) -> None: ...

class ResponseCommit(_message.Message):
    __slots__ = ("retain_height",)
    RETAIN_HEIGHT_FIELD_NUMBER: _ClassVar[int]
    retain_height: int
    def __init__(self, retain_height: _Optional[int] = ...) -> None: ...

class ResponseExtendVote(_message.Message):
    __slots__ = ("vote_extension",)
    VOTE_EXTENSION_FIELD_NUMBER: _ClassVar[int]
    vote_extension: bytes
    def __init__(self, vote_extension: _Optional[bytes] = ...) -> None: ...

class ResponseVerifyVoteExtension(_message.Message):
    __slots__ = ("status",)
    class VerifyStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        UNKNOWN: _ClassVar[ResponseVerifyVoteExtension.VerifyStatus]
        ACCEPT: _ClassVar[ResponseVerifyVoteExtension.VerifyStatus]
        REJECT: _ClassVar[ResponseVerifyVoteExtension.VerifyStatus]
    UNKNOWN: ResponseVerifyVoteExtension.VerifyStatus
    ACCEPT: ResponseVerifyVoteExtension.VerifyStatus
    REJECT: ResponseVerifyVoteExtension.VerifyStatus
    STATUS_FIELD_NUMBER: _ClassVar[int]
    status: ResponseVerifyVoteExtension.VerifyStatus
    def __init__(self, status: _Optional[_Union[ResponseVerifyVoteExtension.VerifyStatus, str]] = ...) -> None: ...

class ResponseFinalizeBlock(_message.Message):
    __slots__ = ("events", "tx_results", "validator_updates", "consensus_param_updates", "app_hash")
    EVENTS_FIELD_NUMBER: _ClassVar[int]
    TX_RESULTS_FIELD_NUMBER: _ClassVar[int]
    VALIDATOR_UPDATES_FIELD_NUMBER: _ClassVar[int]
    CONSENSUS_PARAM_UPDATES_FIELD_NUMBER: _ClassVar[int]
    APP_HASH_FIELD_NUMBER: _ClassVar[int]
    events: _containers.RepeatedCompositeFieldContainer[_types_pb2_1.Event]
    tx_results: _containers.RepeatedCompositeFieldContainer[ExecTxResult]
    validator_updates: _containers.RepeatedCompositeFieldContainer[_types_pb2.ValidatorUpdate]
    consensus_param_updates: _params_pb2.ConsensusParams
    app_hash: bytes
    def __init__(self, events: _Optional[_Iterable[_Union[_types_pb2_1.Event, _Mapping]]] = ..., tx_results: _Optional[_Iterable[_Union[ExecTxResult, _Mapping]]] = ..., validator_updates: _Optional[_Iterable[_Union[_types_pb2.ValidatorUpdate, _Mapping]]] = ..., consensus_param_updates: _Optional[_Union[_params_pb2.ConsensusParams, _Mapping]] = ..., app_hash: _Optional[bytes] = ...) -> None: ...

class VoteInfo(_message.Message):
    __slots__ = ("validator", "block_id_flag")
    VALIDATOR_FIELD_NUMBER: _ClassVar[int]
    BLOCK_ID_FLAG_FIELD_NUMBER: _ClassVar[int]
    validator: _types_pb2.Validator
    block_id_flag: _validator_pb2.BlockIDFlag
    def __init__(self, validator: _Optional[_Union[_types_pb2.Validator, _Mapping]] = ..., block_id_flag: _Optional[_Union[_validator_pb2.BlockIDFlag, str]] = ...) -> None: ...

class ExtendedVoteInfo(_message.Message):
    __slots__ = ("validator", "vote_extension", "extension_signature", "block_id_flag")
    VALIDATOR_FIELD_NUMBER: _ClassVar[int]
    VOTE_EXTENSION_FIELD_NUMBER: _ClassVar[int]
    EXTENSION_SIGNATURE_FIELD_NUMBER: _ClassVar[int]
    BLOCK_ID_FLAG_FIELD_NUMBER: _ClassVar[int]
    validator: _types_pb2.Validator
    vote_extension: bytes
    extension_signature: bytes
    block_id_flag: _validator_pb2.BlockIDFlag
    def __init__(self, validator: _Optional[_Union[_types_pb2.Validator, _Mapping]] = ..., vote_extension: _Optional[bytes] = ..., extension_signature: _Optional[bytes] = ..., block_id_flag: _Optional[_Union[_validator_pb2.BlockIDFlag, str]] = ...) -> None: ...

class CommitInfo(_message.Message):
    __slots__ = ("round", "votes")
    ROUND_FIELD_NUMBER: _ClassVar[int]
    VOTES_FIELD_NUMBER: _ClassVar[int]
    round: int
    votes: _containers.RepeatedCompositeFieldContainer[VoteInfo]
    def __init__(self, round: _Optional[int] = ..., votes: _Optional[_Iterable[_Union[VoteInfo, _Mapping]]] = ...) -> None: ...

class ExtendedCommitInfo(_message.Message):
    __slots__ = ("round", "votes")
    ROUND_FIELD_NUMBER: _ClassVar[int]
    VOTES_FIELD_NUMBER: _ClassVar[int]
    round: int
    votes: _containers.RepeatedCompositeFieldContainer[ExtendedVoteInfo]
    def __init__(self, round: _Optional[int] = ..., votes: _Optional[_Iterable[_Union[ExtendedVoteInfo, _Mapping]]] = ...) -> None: ...

class ExecTxResult(_message.Message):
    __slots__ = ("code", "data", "log", "info", "gas_wanted", "gas_used", "events", "codespace")
    CODE_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    LOG_FIELD_NUMBER: _ClassVar[int]
    INFO_FIELD_NUMBER: _ClassVar[int]
    GAS_WANTED_FIELD_NUMBER: _ClassVar[int]
    GAS_USED_FIELD_NUMBER: _ClassVar[int]
    EVENTS_FIELD_NUMBER: _ClassVar[int]
    CODESPACE_FIELD_NUMBER: _ClassVar[int]
    code: int
    data: bytes
    log: str
    info: str
    gas_wanted: int
    gas_used: int
    events: _containers.RepeatedCompositeFieldContainer[_types_pb2_1.Event]
    codespace: str
    def __init__(self, code: _Optional[int] = ..., data: _Optional[bytes] = ..., log: _Optional[str] = ..., info: _Optional[str] = ..., gas_wanted: _Optional[int] = ..., gas_used: _Optional[int] = ..., events: _Optional[_Iterable[_Union[_types_pb2_1.Event, _Mapping]]] = ..., codespace: _Optional[str] = ...) -> None: ...

class TxResult(_message.Message):
    __slots__ = ("height", "index", "tx", "result")
    HEIGHT_FIELD_NUMBER: _ClassVar[int]
    INDEX_FIELD_NUMBER: _ClassVar[int]
    TX_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    height: int
    index: int
    tx: bytes
    result: ExecTxResult
    def __init__(self, height: _Optional[int] = ..., index: _Optional[int] = ..., tx: _Optional[bytes] = ..., result: _Optional[_Union[ExecTxResult, _Mapping]] = ...) -> None: ...



from cosmos_proto import cosmos_pb2 as _cosmos_pb2
from gogoproto import gogo_pb2 as _gogo_pb2
from google.protobuf import any_pb2 as _any_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class AccessType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    ACCESS_TYPE_UNSPECIFIED: _ClassVar[AccessType]
    ACCESS_TYPE_NOBODY: _ClassVar[AccessType]
    ACCESS_TYPE_ONLY_ADDRESS: _ClassVar[AccessType]
    ACCESS_TYPE_EVERYBODY: _ClassVar[AccessType]

class ContractCodeHistoryOperationType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    CONTRACT_CODE_HISTORY_OPERATION_TYPE_UNSPECIFIED: _ClassVar[ContractCodeHistoryOperationType]
    CONTRACT_CODE_HISTORY_OPERATION_TYPE_INIT: _ClassVar[ContractCodeHistoryOperationType]
    CONTRACT_CODE_HISTORY_OPERATION_TYPE_MIGRATE: _ClassVar[ContractCodeHistoryOperationType]
    CONTRACT_CODE_HISTORY_OPERATION_TYPE_GENESIS: _ClassVar[ContractCodeHistoryOperationType]
ACCESS_TYPE_UNSPECIFIED: AccessType
ACCESS_TYPE_NOBODY: AccessType
ACCESS_TYPE_ONLY_ADDRESS: AccessType
ACCESS_TYPE_EVERYBODY: AccessType
CONTRACT_CODE_HISTORY_OPERATION_TYPE_UNSPECIFIED: ContractCodeHistoryOperationType
CONTRACT_CODE_HISTORY_OPERATION_TYPE_INIT: ContractCodeHistoryOperationType
CONTRACT_CODE_HISTORY_OPERATION_TYPE_MIGRATE: ContractCodeHistoryOperationType
CONTRACT_CODE_HISTORY_OPERATION_TYPE_GENESIS: ContractCodeHistoryOperationType

class AccessTypeParam(_message.Message):
    __slots__ = ("value",)
    VALUE_FIELD_NUMBER: _ClassVar[int]
    value: AccessType
    def __init__(self, value: _Optional[_Union[AccessType, str]] = ...) -> None: ...

class AccessConfig(_message.Message):
    __slots__ = ("permission", "address")
    PERMISSION_FIELD_NUMBER: _ClassVar[int]
    ADDRESS_FIELD_NUMBER: _ClassVar[int]
    permission: AccessType
    address: str
    def __init__(self, permission: _Optional[_Union[AccessType, str]] = ..., address: _Optional[str] = ...) -> None: ...

class Params(_message.Message):
    __slots__ = ("code_upload_access", "instantiate_default_permission")
    CODE_UPLOAD_ACCESS_FIELD_NUMBER: _ClassVar[int]
    INSTANTIATE_DEFAULT_PERMISSION_FIELD_NUMBER: _ClassVar[int]
    code_upload_access: AccessConfig
    instantiate_default_permission: AccessType
    def __init__(self, code_upload_access: _Optional[_Union[AccessConfig, _Mapping]] = ..., instantiate_default_permission: _Optional[_Union[AccessType, str]] = ...) -> None: ...

class CodeInfo(_message.Message):
    __slots__ = ("code_hash", "creator", "instantiate_config")
    CODE_HASH_FIELD_NUMBER: _ClassVar[int]
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    INSTANTIATE_CONFIG_FIELD_NUMBER: _ClassVar[int]
    code_hash: bytes
    creator: str
    instantiate_config: AccessConfig
    def __init__(self, code_hash: _Optional[bytes] = ..., creator: _Optional[str] = ..., instantiate_config: _Optional[_Union[AccessConfig, _Mapping]] = ...) -> None: ...

class ContractInfo(_message.Message):
    __slots__ = ("code_id", "creator", "admin", "label", "created", "ibc_port_id", "extension")
    CODE_ID_FIELD_NUMBER: _ClassVar[int]
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    ADMIN_FIELD_NUMBER: _ClassVar[int]
    LABEL_FIELD_NUMBER: _ClassVar[int]
    CREATED_FIELD_NUMBER: _ClassVar[int]
    IBC_PORT_ID_FIELD_NUMBER: _ClassVar[int]
    EXTENSION_FIELD_NUMBER: _ClassVar[int]
    code_id: int
    creator: str
    admin: str
    label: str
    created: AbsoluteTxPosition
    ibc_port_id: str
    extension: _any_pb2.Any
    def __init__(self, code_id: _Optional[int] = ..., creator: _Optional[str] = ..., admin: _Optional[str] = ..., label: _Optional[str] = ..., created: _Optional[_Union[AbsoluteTxPosition, _Mapping]] = ..., ibc_port_id: _Optional[str] = ..., extension: _Optional[_Union[_any_pb2.Any, _Mapping]] = ...) -> None: ...

class ContractCodeHistoryEntry(_message.Message):
    __slots__ = ("operation", "code_id", "updated", "msg")
    OPERATION_FIELD_NUMBER: _ClassVar[int]
    CODE_ID_FIELD_NUMBER: _ClassVar[int]
    UPDATED_FIELD_NUMBER: _ClassVar[int]
    MSG_FIELD_NUMBER: _ClassVar[int]
    operation: ContractCodeHistoryOperationType
    code_id: int
    updated: AbsoluteTxPosition
    msg: bytes
    def __init__(self, operation: _Optional[_Union[ContractCodeHistoryOperationType, str]] = ..., code_id: _Optional[int] = ..., updated: _Optional[_Union[AbsoluteTxPosition, _Mapping]] = ..., msg: _Optional[bytes] = ...) -> None: ...

class AbsoluteTxPosition(_message.Message):
    __slots__ = ("block_height", "tx_index")
    BLOCK_HEIGHT_FIELD_NUMBER: _ClassVar[int]
    TX_INDEX_FIELD_NUMBER: _ClassVar[int]
    block_height: int
    tx_index: int
    def __init__(self, block_height: _Optional[int] = ..., tx_index: _Optional[int] = ...) -> None: ...

class Model(_message.Message):
    __slots__ = ("key", "value")
    KEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    key: bytes
    value: bytes
    def __init__(self, key: _Optional[bytes] = ..., value: _Optional[bytes] = ...) -> None: ...



import re
from dataclasses import dataclass
from datetime import datetime, timedelta
from typing import Dict, List, Optional, Union

from neurionpy.synapse.exceptions import (
    BroadcastError,
    InsufficientFeesError,
    OutOfGasError,
)
from neurionpy.crypto.address import Address


@dataclass
class MessageLog:
    """Message Log."""

    index: int  # noqa
    log: str  # noqa
    events: Dict[str, Dict[str, str]]


@dataclass
class TxResponse:
    """Transaction response.

    :raises OutOfGasError: Out of gas error
    :raises InsufficientFeesError: Insufficient fees
    :raises BroadcastError: Broadcast Exception
    """

    hash: str
    height: int
    code: int
    gas_wanted: int
    gas_used: int
    raw_log: str
    logs: List[MessageLog]
    events: Dict[str, Dict[str, str]]
    timestamp: Optional[datetime]

    def is_successful(self) -> bool:
        """Check transaction is successful.

        :return: transaction status
        """
        return self.code == 0

    def ensure_successful(self):
        """Ensure transaction is successful.

        :raises OutOfGasError: Out of gas error
        :raises InsufficientFeesError: Insufficient fees
        :raises BroadcastError: Broadcast Exception
        """
        if self.code != 0:
            if "out of gas" in self.raw_log:
                match = re.search(
                    r"gasWanted:\s*(\d+).*?gasUsed:\s*(\d+)", self.raw_log
                )
                if match is not None:
                    gas_wanted = int(match.group(1))
                    gas_used = int(match.group(2))
                else:
                    gas_wanted = -1
                    gas_used = -1

                raise OutOfGasError(self.hash, gas_wanted=gas_wanted, gas_used=gas_used)
            if "insufficient fees" in self.raw_log:
                match = re.search(r"required:\s*(\d+\w+)", self.raw_log)
                if match is not None:
                    required_fee = match.group(1)
                else:
                    required_fee = f"more than {self.gas_wanted}"
                raise InsufficientFeesError(self.hash, required_fee)
            raise BroadcastError(self.hash, self.raw_log)


class SubmittedTx:
    """Submitted transaction."""

    def __init__(
        self, client: "LedgerClient", tx_hash: str  # type: ignore # noqa: F821
    ):
        """Init the Submitted transaction.

        :param client: Ledger client
        :param tx_hash: transaction hash
        """
        self._client = client
        self._response: Optional[TxResponse] = None
        self._tx_hash = str(tx_hash)

    @property
    def tx_hash(self) -> str:
        """Get the transaction hash.

        :return: transaction hash
        """
        return self._tx_hash

    @property
    def response(self) -> Optional[TxResponse]:
        """Get the transaction response.

        :return: response
        """
        return self._response

    @property
    def contract_code_id(self) -> Optional[int]:
        """Get the contract code id.

        :return: return contract code id if exist else None
        """
        if self._response is None:
            return None

        code_id = self._response.events.get("store_code", {}).get("code_id")
        if code_id is None:
            return None

        return int(code_id)

    @property
    def contract_address(self) -> Optional[Address]:
        """Get the contract address.

        :return: return contract address if exist else None
        """
        if self._response is None:
            return None

        contract_address = self._response.events.get("instantiate", {}).get(
            "_contract_address"
        )
        if contract_address is None:
            return None

        return Address(contract_address)

    def wait_to_complete(
        self,
        timeout: Optional[Union[int, float, timedelta]] = None,
        poll_period: Optional[Union[int, float, timedelta]] = None,
    ) -> "SubmittedTx":
        """Wait to complete the transaction.

        :param timeout: timeout, defaults to None
        :param poll_period: poll_period, defaults to None

        :return: Submitted Transaction
        """
        self._response = self._client.wait_for_query_tx(
            self.tx_hash, timeout=timeout, poll_period=poll_period
        )
        assert self._response is not None
        self._response.ensure_successful()

        return self



from amino import amino_pb2 as _amino_pb2
from cosmos.msg.v1 import msg_pb2 as _msg_pb2
from cosmos_proto import cosmos_pb2 as _cosmos_pb2
from gogoproto import gogo_pb2 as _gogo_pb2
from neurion.crucible import params_pb2 as _params_pb2
from neurion.crucible import task_pb2 as _task_pb2
from neurion.crucible import submission_pb2 as _submission_pb2
from neurion.crucible import score_task_pb2 as _score_task_pb2
from neurion.crucible import plagiarism_report_pb2 as _plagiarism_report_pb2
from neurion.crucible import dispute_score_task_pb2 as _dispute_score_task_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class MsgUpdateParams(_message.Message):
    __slots__ = ("authority", "params")
    AUTHORITY_FIELD_NUMBER: _ClassVar[int]
    PARAMS_FIELD_NUMBER: _ClassVar[int]
    authority: str
    params: _params_pb2.Params
    def __init__(self, authority: _Optional[str] = ..., params: _Optional[_Union[_params_pb2.Params, _Mapping]] = ...) -> None: ...

class MsgUpdateParamsResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgApplyCreator(_message.Message):
    __slots__ = ("creator", "description")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    creator: str
    description: str
    def __init__(self, creator: _Optional[str] = ..., description: _Optional[str] = ...) -> None: ...

class MsgApplyCreatorResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgApproveApplication(_message.Message):
    __slots__ = ("creator", "application_id", "reason")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    APPLICATION_ID_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    creator: str
    application_id: int
    reason: str
    def __init__(self, creator: _Optional[str] = ..., application_id: _Optional[int] = ..., reason: _Optional[str] = ...) -> None: ...

class MsgApproveApplicationResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgRejectApplication(_message.Message):
    __slots__ = ("creator", "application_id", "reason")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    APPLICATION_ID_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    creator: str
    application_id: int
    reason: str
    def __init__(self, creator: _Optional[str] = ..., application_id: _Optional[int] = ..., reason: _Optional[str] = ...) -> None: ...

class MsgRejectApplicationResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgCreateTask(_message.Message):
    __slots__ = ("creator", "reward", "training_kit", "validation_kit", "duration_in_days", "name", "description")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    REWARD_FIELD_NUMBER: _ClassVar[int]
    TRAINING_KIT_FIELD_NUMBER: _ClassVar[int]
    VALIDATION_KIT_FIELD_NUMBER: _ClassVar[int]
    DURATION_IN_DAYS_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    DESCRIPTION_FIELD_NUMBER: _ClassVar[int]
    creator: str
    reward: int
    training_kit: str
    validation_kit: str
    duration_in_days: int
    name: str
    description: str
    def __init__(self, creator: _Optional[str] = ..., reward: _Optional[int] = ..., training_kit: _Optional[str] = ..., validation_kit: _Optional[str] = ..., duration_in_days: _Optional[int] = ..., name: _Optional[str] = ..., description: _Optional[str] = ...) -> None: ...

class MsgCreateTaskResponse(_message.Message):
    __slots__ = ("task",)
    TASK_FIELD_NUMBER: _ClassVar[int]
    task: _task_pb2.Task
    def __init__(self, task: _Optional[_Union[_task_pb2.Task, _Mapping]] = ...) -> None: ...

class MsgRegisterTrainer(_message.Message):
    __slots__ = ("creator", "stake", "task_id")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    STAKE_FIELD_NUMBER: _ClassVar[int]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    creator: str
    stake: int
    task_id: int
    def __init__(self, creator: _Optional[str] = ..., stake: _Optional[int] = ..., task_id: _Optional[int] = ...) -> None: ...

class MsgRegisterTrainerResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgRegisterScorer(_message.Message):
    __slots__ = ("creator", "stake", "task_id")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    STAKE_FIELD_NUMBER: _ClassVar[int]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    creator: str
    stake: int
    task_id: int
    def __init__(self, creator: _Optional[str] = ..., stake: _Optional[int] = ..., task_id: _Optional[int] = ...) -> None: ...

class MsgRegisterScorerResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgSubmitTrainingResult(_message.Message):
    __slots__ = ("creator", "task_id", "result", "metainfo", "encrypted_proof_of_ownership")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    METAINFO_FIELD_NUMBER: _ClassVar[int]
    ENCRYPTED_PROOF_OF_OWNERSHIP_FIELD_NUMBER: _ClassVar[int]
    creator: str
    task_id: int
    result: str
    metainfo: str
    encrypted_proof_of_ownership: str
    def __init__(self, creator: _Optional[str] = ..., task_id: _Optional[int] = ..., result: _Optional[str] = ..., metainfo: _Optional[str] = ..., encrypted_proof_of_ownership: _Optional[str] = ...) -> None: ...

class MsgSubmitTrainingResultResponse(_message.Message):
    __slots__ = ("submission",)
    SUBMISSION_FIELD_NUMBER: _ClassVar[int]
    submission: _submission_pb2.Submission
    def __init__(self, submission: _Optional[_Union[_submission_pb2.Submission, _Mapping]] = ...) -> None: ...

class MsgStakeToTask(_message.Message):
    __slots__ = ("creator", "task_id", "amount")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    AMOUNT_FIELD_NUMBER: _ClassVar[int]
    creator: str
    task_id: int
    amount: int
    def __init__(self, creator: _Optional[str] = ..., task_id: _Optional[int] = ..., amount: _Optional[int] = ...) -> None: ...

class MsgStakeToTaskResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgRequestScoringTask(_message.Message):
    __slots__ = ("creator", "task_id")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    creator: str
    task_id: int
    def __init__(self, creator: _Optional[str] = ..., task_id: _Optional[int] = ...) -> None: ...

class MsgRequestScoringTaskResponse(_message.Message):
    __slots__ = ("score_task",)
    SCORE_TASK_FIELD_NUMBER: _ClassVar[int]
    score_task: _score_task_pb2.ScoreTask
    def __init__(self, score_task: _Optional[_Union[_score_task_pb2.ScoreTask, _Mapping]] = ...) -> None: ...

class MsgSubmitScore(_message.Message):
    __slots__ = ("creator", "score_task_id", "score")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    SCORE_TASK_ID_FIELD_NUMBER: _ClassVar[int]
    SCORE_FIELD_NUMBER: _ClassVar[int]
    creator: str
    score_task_id: int
    score: str
    def __init__(self, creator: _Optional[str] = ..., score_task_id: _Optional[int] = ..., score: _Optional[str] = ...) -> None: ...

class MsgSubmitScoreResponse(_message.Message):
    __slots__ = ("score_task",)
    SCORE_TASK_FIELD_NUMBER: _ClassVar[int]
    score_task: _score_task_pb2.ScoreTask
    def __init__(self, score_task: _Optional[_Union[_score_task_pb2.ScoreTask, _Mapping]] = ...) -> None: ...

class MsgSubmitFinalResult(_message.Message):
    __slots__ = ("creator", "task_id", "result", "metainfo", "encrypted_proof_of_ownership")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    RESULT_FIELD_NUMBER: _ClassVar[int]
    METAINFO_FIELD_NUMBER: _ClassVar[int]
    ENCRYPTED_PROOF_OF_OWNERSHIP_FIELD_NUMBER: _ClassVar[int]
    creator: str
    task_id: int
    result: str
    metainfo: str
    encrypted_proof_of_ownership: str
    def __init__(self, creator: _Optional[str] = ..., task_id: _Optional[int] = ..., result: _Optional[str] = ..., metainfo: _Optional[str] = ..., encrypted_proof_of_ownership: _Optional[str] = ...) -> None: ...

class MsgSubmitFinalResultResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgReportModelPlagiarism(_message.Message):
    __slots__ = ("creator", "my_submission_id", "suspected_submission_id", "encrypt_key")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    MY_SUBMISSION_ID_FIELD_NUMBER: _ClassVar[int]
    SUSPECTED_SUBMISSION_ID_FIELD_NUMBER: _ClassVar[int]
    ENCRYPT_KEY_FIELD_NUMBER: _ClassVar[int]
    creator: str
    my_submission_id: int
    suspected_submission_id: int
    encrypt_key: str
    def __init__(self, creator: _Optional[str] = ..., my_submission_id: _Optional[int] = ..., suspected_submission_id: _Optional[int] = ..., encrypt_key: _Optional[str] = ...) -> None: ...

class MsgReportModelPlagiarismResponse(_message.Message):
    __slots__ = ("report",)
    REPORT_FIELD_NUMBER: _ClassVar[int]
    report: _plagiarism_report_pb2.PlagiarismReport
    def __init__(self, report: _Optional[_Union[_plagiarism_report_pb2.PlagiarismReport, _Mapping]] = ...) -> None: ...

class MsgAcceptPlagiarismReport(_message.Message):
    __slots__ = ("creator", "report_id", "proof_of_plagiarism")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    REPORT_ID_FIELD_NUMBER: _ClassVar[int]
    PROOF_OF_PLAGIARISM_FIELD_NUMBER: _ClassVar[int]
    creator: str
    report_id: int
    proof_of_plagiarism: str
    def __init__(self, creator: _Optional[str] = ..., report_id: _Optional[int] = ..., proof_of_plagiarism: _Optional[str] = ...) -> None: ...

class MsgAcceptPlagiarismReportResponse(_message.Message):
    __slots__ = ("report",)
    REPORT_FIELD_NUMBER: _ClassVar[int]
    report: _plagiarism_report_pb2.PlagiarismReport
    def __init__(self, report: _Optional[_Union[_plagiarism_report_pb2.PlagiarismReport, _Mapping]] = ...) -> None: ...

class MsgRejectPlagiarismReport(_message.Message):
    __slots__ = ("creator", "report_id", "reason")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    REPORT_ID_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    creator: str
    report_id: int
    reason: str
    def __init__(self, creator: _Optional[str] = ..., report_id: _Optional[int] = ..., reason: _Optional[str] = ...) -> None: ...

class MsgRejectPlagiarismReportResponse(_message.Message):
    __slots__ = ("report",)
    REPORT_FIELD_NUMBER: _ClassVar[int]
    report: _plagiarism_report_pb2.PlagiarismReport
    def __init__(self, report: _Optional[_Union[_plagiarism_report_pb2.PlagiarismReport, _Mapping]] = ...) -> None: ...

class MsgDisputeSubmissionScore(_message.Message):
    __slots__ = ("creator", "submission_id", "score")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    SUBMISSION_ID_FIELD_NUMBER: _ClassVar[int]
    SCORE_FIELD_NUMBER: _ClassVar[int]
    creator: str
    submission_id: int
    score: str
    def __init__(self, creator: _Optional[str] = ..., submission_id: _Optional[int] = ..., score: _Optional[str] = ...) -> None: ...

class MsgDisputeSubmissionScoreResponse(_message.Message):
    __slots__ = ("dispute_score_task",)
    DISPUTE_SCORE_TASK_FIELD_NUMBER: _ClassVar[int]
    dispute_score_task: _dispute_score_task_pb2.DisputeScoreTask
    def __init__(self, dispute_score_task: _Optional[_Union[_dispute_score_task_pb2.DisputeScoreTask, _Mapping]] = ...) -> None: ...

class MsgStartTask(_message.Message):
    __slots__ = ("creator", "task_id")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    creator: str
    task_id: int
    def __init__(self, creator: _Optional[str] = ..., task_id: _Optional[int] = ...) -> None: ...

class MsgStartTaskResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgAbortTask(_message.Message):
    __slots__ = ("creator", "task_id")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    creator: str
    task_id: int
    def __init__(self, creator: _Optional[str] = ..., task_id: _Optional[int] = ...) -> None: ...

class MsgAbortTaskResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgTriggerTaskToFinalSubmission(_message.Message):
    __slots__ = ("creator", "task_id")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    creator: str
    task_id: int
    def __init__(self, creator: _Optional[str] = ..., task_id: _Optional[int] = ...) -> None: ...

class MsgTriggerTaskToFinalSubmissionResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgTriggerTaskToFinalTesting(_message.Message):
    __slots__ = ("creator", "task_id", "test_kit")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    TEST_KIT_FIELD_NUMBER: _ClassVar[int]
    creator: str
    task_id: int
    test_kit: str
    def __init__(self, creator: _Optional[str] = ..., task_id: _Optional[int] = ..., test_kit: _Optional[str] = ...) -> None: ...

class MsgTriggerTaskToFinalTestingResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgTerminateTask(_message.Message):
    __slots__ = ("creator", "task_id")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    creator: str
    task_id: int
    def __init__(self, creator: _Optional[str] = ..., task_id: _Optional[int] = ...) -> None: ...

class MsgTerminateTaskResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgClaimTaskReward(_message.Message):
    __slots__ = ("creator", "task_id")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    creator: str
    task_id: int
    def __init__(self, creator: _Optional[str] = ..., task_id: _Optional[int] = ...) -> None: ...

class MsgClaimTaskRewardResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgUnstakeFromTask(_message.Message):
    __slots__ = ("creator", "task_id")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    TASK_ID_FIELD_NUMBER: _ClassVar[int]
    creator: str
    task_id: int
    def __init__(self, creator: _Optional[str] = ..., task_id: _Optional[int] = ...) -> None: ...

class MsgUnstakeFromTaskResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgDisclaimCreatorStatus(_message.Message):
    __slots__ = ("creator", "reason")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    creator: str
    reason: str
    def __init__(self, creator: _Optional[str] = ..., reason: _Optional[str] = ...) -> None: ...

class MsgDisclaimCreatorStatusResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: cosmos/capability/v1beta1/capability.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'cosmos/capability/v1beta1/capability.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*cosmos/capability/v1beta1/capability.proto\x12\x19\x63osmos.capability.v1beta1\x1a\x14gogoproto/gogo.proto\"3\n\nCapability\x12\x1f\n\x05index\x18\x01 \x01(\x04\x42\x10\xf2\xde\x1f\x0cyaml:\"index\":\x04\x98\xa0\x1f\x00\"S\n\x05Owner\x12!\n\x06module\x18\x01 \x01(\tB\x11\xf2\xde\x1f\ryaml:\"module\"\x12\x1d\n\x04name\x18\x02 \x01(\tB\x0f\xf2\xde\x1f\x0byaml:\"name\":\x08\x88\xa0\x1f\x00\x98\xa0\x1f\x00\"J\n\x10\x43\x61pabilityOwners\x12\x36\n\x06owners\x18\x01 \x03(\x0b\x32 .cosmos.capability.v1beta1.OwnerB\x04\xc8\xde\x1f\x00\x42\x31Z/github.com/cosmos/cosmos-sdk/x/capability/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cosmos.capability.v1beta1.capability_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z/github.com/cosmos/cosmos-sdk/x/capability/types'
  _globals['_CAPABILITY'].fields_by_name['index']._loaded_options = None
  _globals['_CAPABILITY'].fields_by_name['index']._serialized_options = b'\362\336\037\014yaml:\"index\"'
  _globals['_CAPABILITY']._loaded_options = None
  _globals['_CAPABILITY']._serialized_options = b'\230\240\037\000'
  _globals['_OWNER'].fields_by_name['module']._loaded_options = None
  _globals['_OWNER'].fields_by_name['module']._serialized_options = b'\362\336\037\ryaml:\"module\"'
  _globals['_OWNER'].fields_by_name['name']._loaded_options = None
  _globals['_OWNER'].fields_by_name['name']._serialized_options = b'\362\336\037\013yaml:\"name\"'
  _globals['_OWNER']._loaded_options = None
  _globals['_OWNER']._serialized_options = b'\210\240\037\000\230\240\037\000'
  _globals['_CAPABILITYOWNERS'].fields_by_name['owners']._loaded_options = None
  _globals['_CAPABILITYOWNERS'].fields_by_name['owners']._serialized_options = b'\310\336\037\000'
  _globals['_CAPABILITY']._serialized_start=95
  _globals['_CAPABILITY']._serialized_end=146
  _globals['_OWNER']._serialized_start=148
  _globals['_OWNER']._serialized_end=231
  _globals['_CAPABILITYOWNERS']._serialized_start=233
  _globals['_CAPABILITYOWNERS']._serialized_end=307
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tendermint/rpc/grpc/types.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'tendermint/rpc/grpc/types.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tendermint.abci import types_pb2 as tendermint_dot_abci_dot_types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1ftendermint/rpc/grpc/types.proto\x12\x13tendermint.rpc.grpc\x1a\x1btendermint/abci/types.proto\"\r\n\x0bRequestPing\" \n\x12RequestBroadcastTx\x12\n\n\x02tx\x18\x01 \x01(\x0c\"\x0e\n\x0cResponsePing\"\x81\x01\n\x13ResponseBroadcastTx\x12\x32\n\x08\x63heck_tx\x18\x01 \x01(\x0b\x32 .tendermint.abci.ResponseCheckTx\x12\x36\n\ndeliver_tx\x18\x02 \x01(\x0b\x32\".tendermint.abci.ResponseDeliverTx2\xbd\x01\n\x0c\x42roadcastAPI\x12K\n\x04Ping\x12 .tendermint.rpc.grpc.RequestPing\x1a!.tendermint.rpc.grpc.ResponsePing\x12`\n\x0b\x42roadcastTx\x12\'.tendermint.rpc.grpc.RequestBroadcastTx\x1a(.tendermint.rpc.grpc.ResponseBroadcastTxB4Z2github.com/tendermint/tendermint/rpc/grpc;coregrpcb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tendermint.rpc.grpc.types_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z2github.com/tendermint/tendermint/rpc/grpc;coregrpc'
  _globals['_REQUESTPING']._serialized_start=85
  _globals['_REQUESTPING']._serialized_end=98
  _globals['_REQUESTBROADCASTTX']._serialized_start=100
  _globals['_REQUESTBROADCASTTX']._serialized_end=132
  _globals['_RESPONSEPING']._serialized_start=134
  _globals['_RESPONSEPING']._serialized_end=148
  _globals['_RESPONSEBROADCASTTX']._serialized_start=151
  _globals['_RESPONSEBROADCASTTX']._serialized_end=280
  _globals['_BROADCASTAPI']._serialized_start=283
  _globals['_BROADCASTAPI']._serialized_end=472
# @@protoc_insertion_point(module_scope)



from cometbft.services.pruning.v1 import pruning_pb2 as _pruning_pb2
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor



from abc import ABC, abstractmethod
from typing import Dict, Optional

from neurionpy.synapse.tx import Transaction
from neurionpy.protos.cosmos.consensus.v1.query_pb2 import QueryParamsRequest


class GasStrategy(ABC):
    """Transaction gas strategy."""

    @abstractmethod
    def estimate_gas(self, tx: Transaction) -> int:
        """Estimate the transaction gas.

        :param tx: Transaction
        :return: None
        """

    @abstractmethod
    def block_gas_limit(self) -> int:
        """Get the block gas limit.

        :return: None
        """

    def _clip_gas(self, value: int) -> int:
        block_limit = self.block_gas_limit()
        if block_limit < 0:
            return value

        return min(value, block_limit)


class SimulationGasStrategy(GasStrategy):
    """Simulation transaction gas strategy.

    :param GasStrategy: gas strategy
    """

    DEFAULT_MULTIPLIER = 1.65

    def __init__(self, client: "LedgerClient", multiplier: Optional[float] = None):  # type: ignore # noqa: F821
        """Init the Simulation transaction gas strategy.

        :param client: Ledger client
        :param multiplier: multiplier, defaults to None
        """
        self._client = client
        self._max_gas: Optional[int] = None
        self._multiplier = multiplier or self.DEFAULT_MULTIPLIER

    def estimate_gas(self, tx: Transaction) -> int:
        """Get estimated transaction gas.

        :param tx: transaction
        :return: Estimated transaction gas
        """
        gas_estimate = self._client.simulate_tx(tx)
        return self._clip_gas(int(gas_estimate * self._multiplier))

    def block_gas_limit(self) -> int:
        """Get the block gas limit.

        :return: block gas limit
        """
        if self._max_gas is None:
            consensus_params = self._client.consensus.Params(QueryParamsRequest())
            self._max_gas = int(consensus_params.params.block.max_gas)

        return self._max_gas or -1


class OfflineMessageTableStrategy(GasStrategy):
    """Offline message table strategy.

    :param GasStrategy: gas strategy
    """

    DEFAULT_FALLBACK_GAS_LIMIT = 400_000
    DEFAULT_BLOCK_LIMIT = 2_000_000

    @staticmethod
    def default_table() -> "OfflineMessageTableStrategy":
        """offline message strategy default table.

        :return: offline message default table strategy
        """
        strategy = OfflineMessageTableStrategy()
        strategy.update_entry("cosmos.bank.v1beta1.MsgSend", 100_000)
        strategy.update_entry("cosmwasm.wasm.v1.MsgStoreCode", 2_000_000)
        strategy.update_entry("cosmwasm.wasm.v1.MsgInstantiateContract", 250_000)
        strategy.update_entry("cosmwasm.wasm.v1.MsgExecuteContract", 400_000)
        return strategy

    def __init__(
        self,
        fallback_gas_limit: Optional[int] = None,
        block_limit: Optional[int] = None,
    ):
        """Init offline message table strategy.

        :param fallback_gas_limit: Fallback gas limit, defaults to None
        :param block_limit: Block limit, defaults to None
        """
        self._table: Dict[str, int] = {}
        self._block_limit = block_limit or self.DEFAULT_BLOCK_LIMIT
        self._fallback_gas_limit = fallback_gas_limit or self.DEFAULT_FALLBACK_GAS_LIMIT

    def update_entry(self, transaction_type: str, gas_limit: int):
        """Update the entry of the transaction.

        :param transaction_type: transaction type
        :param gas_limit: gas limit
        """
        self._table[str(transaction_type)] = int(gas_limit)

    def estimate_gas(self, tx: Transaction) -> int:
        """Get estimated transaction gas.

        :param tx: transaction
        :return: Estimated transaction gas
        """
        gas_estimate = 0
        for msg in tx.msgs:
            gas_estimate += self._table.get(
                msg.DESCRIPTOR.full_name, self._fallback_gas_limit
            )
        return self._clip_gas(gas_estimate)

    def block_gas_limit(self) -> int:
        """Get the block gas limit.

        :return: block gas limit
        """
        return self._block_limit



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: neurion/sanctum/dataset_application.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'neurion/sanctum/dataset_application.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n)neurion/sanctum/dataset_application.proto\x12\x0fneurion.sanctum\"\xf7\x02\n\x12\x44\x61tasetApplication\x12\n\n\x02id\x18\x01 \x01(\x04\x12\x0f\n\x07\x63reator\x18\x02 \x01(\t\x12\x32\n\x06result\x18\x03 \x01(\x0e\x32\".neurion.sanctum.ApplicationResult\x12\x1a\n\x12\x64\x65\x63ision_timestamp\x18\x04 \x01(\x04\x12\x16\n\x0e\x64\x65\x63ision_maker\x18\x05 \x01(\t\x12\x17\n\x0f\x64\x65\x63ision_reason\x18\x06 \x01(\t\x12\r\n\x05stake\x18\x07 \x01(\x04\x12\x1a\n\x12\x63reation_timestamp\x18\x08 \x01(\x04\x12\x1b\n\x13\x65ncrypted_data_link\x18\t \x01(\t\x12\x18\n\x10\x65xplanation_link\x18\n \x01(\t\x12\x0f\n\x07\x63ontact\x18\x0b \x01(\t\x12\x1d\n\x15proof_of_authenticity\x18\x0c \x01(\t\x12\x19\n\x11\x64\x61taset_usage_fee\x18\r \x01(\x04\x12\x16\n\x0e\x64isputed_count\x18\x0e \x01(\x04*\x9d\x01\n\x11\x41pplicationResult\x12$\n APPLICATION_RESULT_NOT_AVAILABLE\x10\x00\x12\x1f\n\x1b\x41PPLICATION_RESULT_ACCEPTED\x10\x01\x12\x1f\n\x1b\x41PPLICATION_RESULT_REJECTED\x10\x02\x12 \n\x1c\x41PPLICATION_RESULT_RECLAIMED\x10\x03\x42\x19Z\x17neurion/x/sanctum/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'neurion.sanctum.dataset_application_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\027neurion/x/sanctum/types'
  _globals['_APPLICATIONRESULT']._serialized_start=441
  _globals['_APPLICATIONRESULT']._serialized_end=598
  _globals['_DATASETAPPLICATION']._serialized_start=63
  _globals['_DATASETAPPLICATION']._serialized_end=438
# @@protoc_insertion_point(module_scope)



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class PageRequest(_message.Message):
    __slots__ = ("key", "offset", "limit", "count_total", "reverse")
    KEY_FIELD_NUMBER: _ClassVar[int]
    OFFSET_FIELD_NUMBER: _ClassVar[int]
    LIMIT_FIELD_NUMBER: _ClassVar[int]
    COUNT_TOTAL_FIELD_NUMBER: _ClassVar[int]
    REVERSE_FIELD_NUMBER: _ClassVar[int]
    key: bytes
    offset: int
    limit: int
    count_total: bool
    reverse: bool
    def __init__(self, key: _Optional[bytes] = ..., offset: _Optional[int] = ..., limit: _Optional[int] = ..., count_total: bool = ..., reverse: bool = ...) -> None: ...

class PageResponse(_message.Message):
    __slots__ = ("next_key", "total")
    NEXT_KEY_FIELD_NUMBER: _ClassVar[int]
    TOTAL_FIELD_NUMBER: _ClassVar[int]
    next_key: bytes
    total: int
    def __init__(self, next_key: _Optional[bytes] = ..., total: _Optional[int] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: cosmos/auth/v1beta1/query.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'cosmos/auth/v1beta1/query.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cosmos.base.query.v1beta1 import pagination_pb2 as cosmos_dot_base_dot_query_dot_v1beta1_dot_pagination__pb2
from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from cosmos.auth.v1beta1 import auth_pb2 as cosmos_dot_auth_dot_v1beta1_dot_auth__pb2
from cosmos_proto import cosmos_pb2 as cosmos__proto_dot_cosmos__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1f\x63osmos/auth/v1beta1/query.proto\x12\x13\x63osmos.auth.v1beta1\x1a*cosmos/base/query/v1beta1/pagination.proto\x1a\x14gogoproto/gogo.proto\x1a\x19google/protobuf/any.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x1e\x63osmos/auth/v1beta1/auth.proto\x1a\x19\x63osmos_proto/cosmos.proto\"R\n\x14QueryAccountsRequest\x12:\n\npagination\x18\x01 \x01(\x0b\x32&.cosmos.base.query.v1beta1.PageRequest\"\x8a\x01\n\x15QueryAccountsResponse\x12\x34\n\x08\x61\x63\x63ounts\x18\x01 \x03(\x0b\x32\x14.google.protobuf.AnyB\x0c\xca\xb4-\x08\x41\x63\x63ountI\x12;\n\npagination\x18\x02 \x01(\x0b\x32\'.cosmos.base.query.v1beta1.PageResponse\"0\n\x13QueryAccountRequest\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t:\x08\x88\xa0\x1f\x00\xe8\xa0\x1f\x00\"K\n\x14QueryAccountResponse\x12\x33\n\x07\x61\x63\x63ount\x18\x01 \x01(\x0b\x32\x14.google.protobuf.AnyB\x0c\xca\xb4-\x08\x41\x63\x63ountI\"\x14\n\x12QueryParamsRequest\"H\n\x13QueryParamsResponse\x12\x31\n\x06params\x18\x01 \x01(\x0b\x32\x1b.cosmos.auth.v1beta1.ParamsB\x04\xc8\xde\x1f\x00\x32\xa7\x03\n\x05Query\x12\x88\x01\n\x08\x41\x63\x63ounts\x12).cosmos.auth.v1beta1.QueryAccountsRequest\x1a*.cosmos.auth.v1beta1.QueryAccountsResponse\"%\x82\xd3\xe4\x93\x02\x1f\x12\x1d/cosmos/auth/v1beta1/accounts\x12\x8f\x01\n\x07\x41\x63\x63ount\x12(.cosmos.auth.v1beta1.QueryAccountRequest\x1a).cosmos.auth.v1beta1.QueryAccountResponse\"/\x82\xd3\xe4\x93\x02)\x12\'/cosmos/auth/v1beta1/accounts/{address}\x12\x80\x01\n\x06Params\x12\'.cosmos.auth.v1beta1.QueryParamsRequest\x1a(.cosmos.auth.v1beta1.QueryParamsResponse\"#\x82\xd3\xe4\x93\x02\x1d\x12\x1b/cosmos/auth/v1beta1/paramsB+Z)github.com/cosmos/cosmos-sdk/x/auth/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cosmos.auth.v1beta1.query_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z)github.com/cosmos/cosmos-sdk/x/auth/types'
  _globals['_QUERYACCOUNTSRESPONSE'].fields_by_name['accounts']._loaded_options = None
  _globals['_QUERYACCOUNTSRESPONSE'].fields_by_name['accounts']._serialized_options = b'\312\264-\010AccountI'
  _globals['_QUERYACCOUNTREQUEST']._loaded_options = None
  _globals['_QUERYACCOUNTREQUEST']._serialized_options = b'\210\240\037\000\350\240\037\000'
  _globals['_QUERYACCOUNTRESPONSE'].fields_by_name['account']._loaded_options = None
  _globals['_QUERYACCOUNTRESPONSE'].fields_by_name['account']._serialized_options = b'\312\264-\010AccountI'
  _globals['_QUERYPARAMSRESPONSE'].fields_by_name['params']._loaded_options = None
  _globals['_QUERYPARAMSRESPONSE'].fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _globals['_QUERY'].methods_by_name['Accounts']._loaded_options = None
  _globals['_QUERY'].methods_by_name['Accounts']._serialized_options = b'\202\323\344\223\002\037\022\035/cosmos/auth/v1beta1/accounts'
  _globals['_QUERY'].methods_by_name['Account']._loaded_options = None
  _globals['_QUERY'].methods_by_name['Account']._serialized_options = b'\202\323\344\223\002)\022\'/cosmos/auth/v1beta1/accounts/{address}'
  _globals['_QUERY'].methods_by_name['Params']._loaded_options = None
  _globals['_QUERY'].methods_by_name['Params']._serialized_options = b'\202\323\344\223\002\035\022\033/cosmos/auth/v1beta1/params'
  _globals['_QUERYACCOUNTSREQUEST']._serialized_start=238
  _globals['_QUERYACCOUNTSREQUEST']._serialized_end=320
  _globals['_QUERYACCOUNTSRESPONSE']._serialized_start=323
  _globals['_QUERYACCOUNTSRESPONSE']._serialized_end=461
  _globals['_QUERYACCOUNTREQUEST']._serialized_start=463
  _globals['_QUERYACCOUNTREQUEST']._serialized_end=511
  _globals['_QUERYACCOUNTRESPONSE']._serialized_start=513
  _globals['_QUERYACCOUNTRESPONSE']._serialized_end=588
  _globals['_QUERYPARAMSREQUEST']._serialized_start=590
  _globals['_QUERYPARAMSREQUEST']._serialized_end=610
  _globals['_QUERYPARAMSRESPONSE']._serialized_start=612
  _globals['_QUERYPARAMSRESPONSE']._serialized_end=684
  _globals['_QUERY']._serialized_start=687
  _globals['_QUERY']._serialized_end=1110
# @@protoc_insertion_point(module_scope)



from amino import amino_pb2 as _amino_pb2
from cosmos.msg.v1 import msg_pb2 as _msg_pb2
from cosmos_proto import cosmos_pb2 as _cosmos_pb2
from gogoproto import gogo_pb2 as _gogo_pb2
from neurion.sanctum import params_pb2 as _params_pb2
from neurion.sanctum import dataset_application_pb2 as _dataset_application_pb2
from neurion.sanctum import dataset_usage_request_pb2 as _dataset_usage_request_pb2
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class MsgUpdateParams(_message.Message):
    __slots__ = ("authority", "params")
    AUTHORITY_FIELD_NUMBER: _ClassVar[int]
    PARAMS_FIELD_NUMBER: _ClassVar[int]
    authority: str
    params: _params_pb2.Params
    def __init__(self, authority: _Optional[str] = ..., params: _Optional[_Union[_params_pb2.Params, _Mapping]] = ...) -> None: ...

class MsgUpdateParamsResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgSubmitDatasetApplication(_message.Message):
    __slots__ = ("creator", "encrypted_data_link", "explanation_link", "contact", "stake", "proof_of_authenticity", "dataset_usage_fee")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    ENCRYPTED_DATA_LINK_FIELD_NUMBER: _ClassVar[int]
    EXPLANATION_LINK_FIELD_NUMBER: _ClassVar[int]
    CONTACT_FIELD_NUMBER: _ClassVar[int]
    STAKE_FIELD_NUMBER: _ClassVar[int]
    PROOF_OF_AUTHENTICITY_FIELD_NUMBER: _ClassVar[int]
    DATASET_USAGE_FEE_FIELD_NUMBER: _ClassVar[int]
    creator: str
    encrypted_data_link: str
    explanation_link: str
    contact: str
    stake: int
    proof_of_authenticity: str
    dataset_usage_fee: int
    def __init__(self, creator: _Optional[str] = ..., encrypted_data_link: _Optional[str] = ..., explanation_link: _Optional[str] = ..., contact: _Optional[str] = ..., stake: _Optional[int] = ..., proof_of_authenticity: _Optional[str] = ..., dataset_usage_fee: _Optional[int] = ...) -> None: ...

class MsgSubmitDatasetApplicationResponse(_message.Message):
    __slots__ = ("dataset_application",)
    DATASET_APPLICATION_FIELD_NUMBER: _ClassVar[int]
    dataset_application: _dataset_application_pb2.DatasetApplication
    def __init__(self, dataset_application: _Optional[_Union[_dataset_application_pb2.DatasetApplication, _Mapping]] = ...) -> None: ...

class MsgApproveApplication(_message.Message):
    __slots__ = ("creator", "application_id", "reason")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    APPLICATION_ID_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    creator: str
    application_id: int
    reason: str
    def __init__(self, creator: _Optional[str] = ..., application_id: _Optional[int] = ..., reason: _Optional[str] = ...) -> None: ...

class MsgApproveApplicationResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgRejectApplication(_message.Message):
    __slots__ = ("creator", "application_id", "reason")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    APPLICATION_ID_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    creator: str
    application_id: int
    reason: str
    def __init__(self, creator: _Optional[str] = ..., application_id: _Optional[int] = ..., reason: _Optional[str] = ...) -> None: ...

class MsgRejectApplicationResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgDisclaimDataset(_message.Message):
    __slots__ = ("creator", "application_id", "reason")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    APPLICATION_ID_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    creator: str
    application_id: int
    reason: str
    def __init__(self, creator: _Optional[str] = ..., application_id: _Optional[int] = ..., reason: _Optional[str] = ...) -> None: ...

class MsgDisclaimDatasetResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgRequestToUseDataset(_message.Message):
    __slots__ = ("creator", "dataset_id", "intent", "training_repository", "contact")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    DATASET_ID_FIELD_NUMBER: _ClassVar[int]
    INTENT_FIELD_NUMBER: _ClassVar[int]
    TRAINING_REPOSITORY_FIELD_NUMBER: _ClassVar[int]
    CONTACT_FIELD_NUMBER: _ClassVar[int]
    creator: str
    dataset_id: int
    intent: str
    training_repository: str
    contact: str
    def __init__(self, creator: _Optional[str] = ..., dataset_id: _Optional[int] = ..., intent: _Optional[str] = ..., training_repository: _Optional[str] = ..., contact: _Optional[str] = ...) -> None: ...

class MsgRequestToUseDatasetResponse(_message.Message):
    __slots__ = ("request",)
    REQUEST_FIELD_NUMBER: _ClassVar[int]
    request: _dataset_usage_request_pb2.DatasetUsageRequest
    def __init__(self, request: _Optional[_Union[_dataset_usage_request_pb2.DatasetUsageRequest, _Mapping]] = ...) -> None: ...

class MsgCancelDatasetUsageRequest(_message.Message):
    __slots__ = ("creator", "request_id", "reason")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    creator: str
    request_id: int
    reason: str
    def __init__(self, creator: _Optional[str] = ..., request_id: _Optional[int] = ..., reason: _Optional[str] = ...) -> None: ...

class MsgCancelDatasetUsageRequestResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgRejectDatasetUsageRequest(_message.Message):
    __slots__ = ("creator", "request_id", "reason")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    creator: str
    request_id: int
    reason: str
    def __init__(self, creator: _Optional[str] = ..., request_id: _Optional[int] = ..., reason: _Optional[str] = ...) -> None: ...

class MsgRejectDatasetUsageRequestResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgApproveDatasetUsageRequest(_message.Message):
    __slots__ = ("creator", "request_id", "reason")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    creator: str
    request_id: int
    reason: str
    def __init__(self, creator: _Optional[str] = ..., request_id: _Optional[int] = ..., reason: _Optional[str] = ...) -> None: ...

class MsgApproveDatasetUsageRequestResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgAddProcessor(_message.Message):
    __slots__ = ("creator", "processor")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    PROCESSOR_FIELD_NUMBER: _ClassVar[int]
    creator: str
    processor: str
    def __init__(self, creator: _Optional[str] = ..., processor: _Optional[str] = ...) -> None: ...

class MsgAddProcessorResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgRemoveProcessor(_message.Message):
    __slots__ = ("creator", "processor")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    PROCESSOR_FIELD_NUMBER: _ClassVar[int]
    creator: str
    processor: str
    def __init__(self, creator: _Optional[str] = ..., processor: _Optional[str] = ...) -> None: ...

class MsgRemoveProcessorResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgProcessDatasetUsageRequest(_message.Message):
    __slots__ = ("creator", "request_id")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    creator: str
    request_id: int
    def __init__(self, creator: _Optional[str] = ..., request_id: _Optional[int] = ...) -> None: ...

class MsgProcessDatasetUsageRequestResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgFinishDatasetUsageRequest(_message.Message):
    __slots__ = ("creator", "request_id", "feedback")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    FEEDBACK_FIELD_NUMBER: _ClassVar[int]
    creator: str
    request_id: int
    feedback: str
    def __init__(self, creator: _Optional[str] = ..., request_id: _Optional[int] = ..., feedback: _Optional[str] = ...) -> None: ...

class MsgFinishDatasetUsageRequestResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgDisputeDatasetUsageRequest(_message.Message):
    __slots__ = ("creator", "request_id", "model", "reason")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    MODEL_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    creator: str
    request_id: int
    model: str
    reason: str
    def __init__(self, creator: _Optional[str] = ..., request_id: _Optional[int] = ..., model: _Optional[str] = ..., reason: _Optional[str] = ...) -> None: ...

class MsgDisputeDatasetUsageRequestResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgApproveDispute(_message.Message):
    __slots__ = ("creator", "request_id", "reason")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    creator: str
    request_id: int
    reason: str
    def __init__(self, creator: _Optional[str] = ..., request_id: _Optional[int] = ..., reason: _Optional[str] = ...) -> None: ...

class MsgApproveDisputeResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgRejectDispute(_message.Message):
    __slots__ = ("creator", "request_id", "reason")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    REQUEST_ID_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    creator: str
    request_id: int
    reason: str
    def __init__(self, creator: _Optional[str] = ..., request_id: _Optional[int] = ..., reason: _Optional[str] = ...) -> None: ...

class MsgRejectDisputeResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgStakeToSanctum(_message.Message):
    __slots__ = ("creator", "amount")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    AMOUNT_FIELD_NUMBER: _ClassVar[int]
    creator: str
    amount: int
    def __init__(self, creator: _Optional[str] = ..., amount: _Optional[int] = ...) -> None: ...

class MsgStakeToSanctumResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgUnstakeFromSanctum(_message.Message):
    __slots__ = ("creator", "amount")
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    AMOUNT_FIELD_NUMBER: _ClassVar[int]
    creator: str
    amount: int
    def __init__(self, creator: _Optional[str] = ..., amount: _Optional[int] = ...) -> None: ...

class MsgUnstakeFromSanctumResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class MsgClaimReward(_message.Message):
    __slots__ = ("creator",)
    CREATOR_FIELD_NUMBER: _ClassVar[int]
    creator: str
    def __init__(self, creator: _Optional[str] = ...) -> None: ...

class MsgClaimRewardResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: ibc/core/client/v1/client.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'ibc/core/client/v1/client.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from cosmos.upgrade.v1beta1 import upgrade_pb2 as cosmos_dot_upgrade_dot_v1beta1_dot_upgrade__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1fibc/core/client/v1/client.proto\x12\x12ibc.core.client.v1\x1a\x14gogoproto/gogo.proto\x1a\x19google/protobuf/any.proto\x1a$cosmos/upgrade/v1beta1/upgrade.proto\"\x85\x01\n\x15IdentifiedClientState\x12\'\n\tclient_id\x18\x01 \x01(\tB\x14\xf2\xde\x1f\x10yaml:\"client_id\"\x12\x43\n\x0c\x63lient_state\x18\x02 \x01(\x0b\x32\x14.google.protobuf.AnyB\x17\xf2\xde\x1f\x13yaml:\"client_state\"\"\x96\x01\n\x18\x43onsensusStateWithHeight\x12\x30\n\x06height\x18\x01 \x01(\x0b\x32\x1a.ibc.core.client.v1.HeightB\x04\xc8\xde\x1f\x00\x12H\n\x0f\x63onsensus_state\x18\x02 \x01(\x0b\x32\x14.google.protobuf.AnyB\x19\xf2\xde\x1f\x15yaml\"consensus_state\"\"\xa9\x01\n\x15\x43lientConsensusStates\x12\'\n\tclient_id\x18\x01 \x01(\tB\x14\xf2\xde\x1f\x10yaml:\"client_id\"\x12g\n\x10\x63onsensus_states\x18\x02 \x03(\x0b\x32,.ibc.core.client.v1.ConsensusStateWithHeightB\x1f\xc8\xde\x1f\x00\xf2\xde\x1f\x17yaml:\"consensus_states\"\"\xb8\x01\n\x14\x43lientUpdateProposal\x12\r\n\x05title\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x37\n\x11subject_client_id\x18\x03 \x01(\tB\x1c\xf2\xde\x1f\x18yaml:\"subject_client_id\"\x12=\n\x14substitute_client_id\x18\x04 \x01(\tB\x1f\xf2\xde\x1f\x1byaml:\"substitute_client_id\":\x04\x88\xa0\x1f\x00\"\xcc\x01\n\x0fUpgradeProposal\x12\r\n\x05title\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x30\n\x04plan\x18\x03 \x01(\x0b\x32\x1c.cosmos.upgrade.v1beta1.PlanB\x04\xc8\xde\x1f\x00\x12U\n\x15upgraded_client_state\x18\x04 \x01(\x0b\x32\x14.google.protobuf.AnyB \xf2\xde\x1f\x1cyaml:\"upgraded_client_state\":\x0c\x88\xa0\x1f\x00\x98\xa0\x1f\x00\xe8\xa0\x1f\x01\"|\n\x06Height\x12\x33\n\x0frevision_number\x18\x01 \x01(\x04\x42\x1a\xf2\xde\x1f\x16yaml:\"revision_number\"\x12\x33\n\x0frevision_height\x18\x02 \x01(\x04\x42\x1a\xf2\xde\x1f\x16yaml:\"revision_height\":\x08\x88\xa0\x1f\x00\x98\xa0\x1f\x00\"=\n\x06Params\x12\x33\n\x0f\x61llowed_clients\x18\x01 \x03(\tB\x1a\xf2\xde\x1f\x16yaml:\"allowed_clients\"B:Z8github.com/cosmos/ibc-go/v2/modules/core/02-client/typesb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ibc.core.client.v1.client_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z8github.com/cosmos/ibc-go/v2/modules/core/02-client/types'
  _globals['_IDENTIFIEDCLIENTSTATE'].fields_by_name['client_id']._loaded_options = None
  _globals['_IDENTIFIEDCLIENTSTATE'].fields_by_name['client_id']._serialized_options = b'\362\336\037\020yaml:\"client_id\"'
  _globals['_IDENTIFIEDCLIENTSTATE'].fields_by_name['client_state']._loaded_options = None
  _globals['_IDENTIFIEDCLIENTSTATE'].fields_by_name['client_state']._serialized_options = b'\362\336\037\023yaml:\"client_state\"'
  _globals['_CONSENSUSSTATEWITHHEIGHT'].fields_by_name['height']._loaded_options = None
  _globals['_CONSENSUSSTATEWITHHEIGHT'].fields_by_name['height']._serialized_options = b'\310\336\037\000'
  _globals['_CONSENSUSSTATEWITHHEIGHT'].fields_by_name['consensus_state']._loaded_options = None
  _globals['_CONSENSUSSTATEWITHHEIGHT'].fields_by_name['consensus_state']._serialized_options = b'\362\336\037\025yaml\"consensus_state\"'
  _globals['_CLIENTCONSENSUSSTATES'].fields_by_name['client_id']._loaded_options = None
  _globals['_CLIENTCONSENSUSSTATES'].fields_by_name['client_id']._serialized_options = b'\362\336\037\020yaml:\"client_id\"'
  _globals['_CLIENTCONSENSUSSTATES'].fields_by_name['consensus_states']._loaded_options = None
  _globals['_CLIENTCONSENSUSSTATES'].fields_by_name['consensus_states']._serialized_options = b'\310\336\037\000\362\336\037\027yaml:\"consensus_states\"'
  _globals['_CLIENTUPDATEPROPOSAL'].fields_by_name['subject_client_id']._loaded_options = None
  _globals['_CLIENTUPDATEPROPOSAL'].fields_by_name['subject_client_id']._serialized_options = b'\362\336\037\030yaml:\"subject_client_id\"'
  _globals['_CLIENTUPDATEPROPOSAL'].fields_by_name['substitute_client_id']._loaded_options = None
  _globals['_CLIENTUPDATEPROPOSAL'].fields_by_name['substitute_client_id']._serialized_options = b'\362\336\037\033yaml:\"substitute_client_id\"'
  _globals['_CLIENTUPDATEPROPOSAL']._loaded_options = None
  _globals['_CLIENTUPDATEPROPOSAL']._serialized_options = b'\210\240\037\000'
  _globals['_UPGRADEPROPOSAL'].fields_by_name['plan']._loaded_options = None
  _globals['_UPGRADEPROPOSAL'].fields_by_name['plan']._serialized_options = b'\310\336\037\000'
  _globals['_UPGRADEPROPOSAL'].fields_by_name['upgraded_client_state']._loaded_options = None
  _globals['_UPGRADEPROPOSAL'].fields_by_name['upgraded_client_state']._serialized_options = b'\362\336\037\034yaml:\"upgraded_client_state\"'
  _globals['_UPGRADEPROPOSAL']._loaded_options = None
  _globals['_UPGRADEPROPOSAL']._serialized_options = b'\210\240\037\000\230\240\037\000\350\240\037\001'
  _globals['_HEIGHT'].fields_by_name['revision_number']._loaded_options = None
  _globals['_HEIGHT'].fields_by_name['revision_number']._serialized_options = b'\362\336\037\026yaml:\"revision_number\"'
  _globals['_HEIGHT'].fields_by_name['revision_height']._loaded_options = None
  _globals['_HEIGHT'].fields_by_name['revision_height']._serialized_options = b'\362\336\037\026yaml:\"revision_height\"'
  _globals['_HEIGHT']._loaded_options = None
  _globals['_HEIGHT']._serialized_options = b'\210\240\037\000\230\240\037\000'
  _globals['_PARAMS'].fields_by_name['allowed_clients']._loaded_options = None
  _globals['_PARAMS'].fields_by_name['allowed_clients']._serialized_options = b'\362\336\037\026yaml:\"allowed_clients\"'
  _globals['_IDENTIFIEDCLIENTSTATE']._serialized_start=143
  _globals['_IDENTIFIEDCLIENTSTATE']._serialized_end=276
  _globals['_CONSENSUSSTATEWITHHEIGHT']._serialized_start=279
  _globals['_CONSENSUSSTATEWITHHEIGHT']._serialized_end=429
  _globals['_CLIENTCONSENSUSSTATES']._serialized_start=432
  _globals['_CLIENTCONSENSUSSTATES']._serialized_end=601
  _globals['_CLIENTUPDATEPROPOSAL']._serialized_start=604
  _globals['_CLIENTUPDATEPROPOSAL']._serialized_end=788
  _globals['_UPGRADEPROPOSAL']._serialized_start=791
  _globals['_UPGRADEPROPOSAL']._serialized_end=995
  _globals['_HEIGHT']._serialized_start=997
  _globals['_HEIGHT']._serialized_end=1121
  _globals['_PARAMS']._serialized_start=1123
  _globals['_PARAMS']._serialized_end=1184
# @@protoc_insertion_point(module_scope)
